/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// REVISION HISTORY:
// Version 2.4 24/09/22;
//						- Changed PDF guide button to link to an online guide.
//						- Added default option for FBX export with version 2020 to settings UI.
//						- Added support for static mesh capsule collision volumes (UCP prefix).
// Version 2.3 17/05/18;
//						- Added support for skeletal meshes to use the Level of Detail (LoD) sets when exporting. Also, skeletal meshes
//							can now be grouped (for convenience) even if not part of a LoD set.
//						- Added a remove LoD button to the settings UI. Allows disassociation with the Level of Detail utility plugin for selected
//							LoD groups.
// Version 2.2 05/05/18;
//						- For batch animation exporting, the selected maxfiles dialog will now close (if open) if the selected maxfiles
//							checkbox is unchecked.
//						- Added support for Level of Detail (LoD) utility for static meshes only, thus render meshes and LOD's can be exported
//							together with collisions (as UE4 supports importing of LOD's if available in a single FBX file).
//						- Added 2017/2018 FBX version to the settings. The default is still 2016, as per UE4 documentation recommendations.
// Version 2.1 07/01/16;
//						- Exporting static meshes now excludes bone geometry.
//						- Added 2016 FBX version to the settings.
//						- Scene must exist as a file before exporting is allowed.
//						- Blendshapes from the Morpher modifier are now exported.
// Version 2.0 03/12/14;
//						- The maxfiles selection rollout will now close if the main UI is closed.
//						- Static meshes and their collisions can now be grouped or linked in the scene and the script will handle it.
// Version 1.9 24/11/14;
//						- Added a new option for exporting static meshes to a single FBX file. The output name will be the maxfile name
//							plus the prefix "StaticMeshes_".
//						- Changed code for exporting files to use a common function where possible for static and skeletal methods.
//						- The scene will now fetch correctly on new exports if previously the skin mesh rollout was closed using the X
//							button.
//						- Adjusted listener output for all functions to include the process completion time.
//						- Added a reset interface settings "x" button next to the Settings "gear icon" button.
// Version 1.8 29/10/14;
//						- Changed the unhide/unfreeze commands to use the doLayer method which avoids the layer warning dialogs.
// Version 1.7 28/10/14;
//						- When exporting skeletal animation where there are multiple hierarchies containing skin meshes and nothing is
//							selected, a dialog will now appear (instead of an error message) to enable a skin mesh to be picked from a list
//							of names. The skeletal export process is then re-run using that selection.
// Version 1.6 27/10/14;
//						- Email change for help contact.
// Version 1.5 22/10/14;
//						- After batch exporting animations the initally opened scene will be reloaded or the scene reset if none was open
//							once processing is comeplete. This makes it non-destructive to the last opened scene due to export processes.
// Version 1.4 17/10/14;
//						- Fixed a bug where exporting a skeletal mesh (non-animation) would use the output animation name.
// Version 1.3 16/10/14;
//						- Added the ability to select from a dropdown list an existing selection set containing static meshes. This function
//							will override the default behaviour of using the current selection, or whole scene export.
//						- Added an editbox to allow the user to specify an animation name for output. Leaving it blank will use the maxfile
//							name as the sequence name (default behaviour). Time tags will override this function, unless Ignore is used.
// Version 1.2 01/10/14;
//						- Current scene exporting (static or skeletal) will now hold the scene before unfreezing all objects, perform the
//							operation then fetch the scene to effectively restore it. This makes it non-destructive to the open scene.
// Version 1.1 30/09/14;
//						- 3ds Max 2015 currently requires all selection attempts to be unfrozen, as it can't select frozen objects (bug).
//							As a result all operations now unfreeze everything in the scene prior to selecting anything.
// Version 1.0 26/09/14;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/

global g_LBTools_FBX

struct s_LBTools_ExportFBX2Unreal
(

/*==================================================================================
// Structure variables
====================================================================================*/

dPos,
f_openExporterUI,
f_openExportSettingsUI,
f_openSelectMaxfilesUI,
exporterINI = getDir #plugcfg + "\LBTools_exporters.ini",
FBXexportINI = getDir #plugcfg + "\LBTools_FBXexport.ini",
exportFBX_Rollout,
FBXsettings_rollout,
maxFileSelection_rollout,
selectSkinMesh_Rollout,
about_Rollout,
imageFileTitle,
str_outputToLog,
str_batchOutputPath = "",
str_batchSourcePath = "",
a_maxfilesToBatch = #(),
a_skinMeshNames = #(),
skinMeshSelectProcess = false,
str_rootPath,

/*==================================================================================
// Structure functions
/*==================================================================================

f_initializeFBXimport
f_initializeFBXexport
f_findSkinModifier
f_selectSkinBones
f_selectHierarchyRoot
f_selectHierarchy
f_makeSubfolder
f_getFilesRecursive
f_getMaxFileNames
f_sortMaxfilesListByDate
f_linearAttach
f_combineArrayIntoString
f_getAllChildren
f_removeLOD
f_createFBXexportINI --// NOTE: This function is called from the LBT_Startup file which should be run before this script interface is opened
f_FBXexportSettingsUI
f_useSameFolder
f_getBatchAnimExportPath
f_batchExportAnimations
f_selectMaxfilesUI
f_exportFBX_Static
f_selectSkinMeshUI
f_exportFBX_Skeletal
f_exportFBX_UI
	
====================================================================================*/

fn f_initializeFBXimport mode:#merge =
(
	fbxImporterSetParam "Animation" true
	fbxImporterSetParam "AxisConversion" false
	fbxImporterSetParam "BakeAnimationLayers" true
	fbxImporterSetParam "Cameras" false
	fbxImporterSetParam "ConvertUnit" "cm"
	fbxImporterSetParam "FillTimeline" false
	fbxImporterSetParam "FilterKeyReducer" false
	fbxImporterSetParam "FilterKeySync" false
	fbxImporterSetParam "GenerateLog" false
	fbxImporterSetParam "ImportBoneAsDummy" true
	fbxImporterSetParam "KeepFrameRate" false
	fbxImporterSetParam "Lights" false
	--fbxImporterSetParam "LoadImportPresetFile"
	fbxImporterSetParam "Markers" false
	--//=============================================================================================================================
	--// Pass the in mode name settings "#merge" (Update), "#create" (Add) or "#exmerge" (Add and Update), as parameters in the call script
	fbxImporterSetParam "Mode" mode
	--//=============================================================================================================================
	--fbxImporterSetParam "PopSettings"
	--fbxImporterSetParam "PushSettings"
	fbxImporterSetParam "PointCache" true
	fbxImporterSetParam "Resampling" 30.0
	--fbxImporterSetParam "ResetImport"
	fbxImporterSetParam "ScaleConversion" true
	fbxImporterSetParam "ScaleFactor" 1.0
	fbxImporterSetParam "Shape" false
	fbxImporterSetParam "Skin" false
	fbxImporterSetParam "SmoothingGroups" false
	--fbxImporterSetParam "TakeCount"
	--fbxImporterSetParam "TakeIndex"
	fbxImporterSetParam "UpAxis" "Y"
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_initializeFBXexport =
(
	fbxExporterSetParam "Animation" true
	fbxExporterSetParam "ASCII" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "ASCII"))
	fbxExporterSetParam "AxisConversionMethod" "Animation"
	fbxExporterSetParam "BakeAnimation" true
	fbxExporterSetParam "BakeframeStart" animationrange.start
	fbxExporterSetParam "BakeframeEnd" animationrange.end
	fbxExporterSetParam "BakeframeStep" 1
	fbxExporterSetParam "BakeResampleAnimation" true
	fbxExporterSetParam "Cameras" false
	fbxExporterSetParam "CAT2HIK" false
	fbxExporterSetParam "ColladaTriangulate" false
	fbxExporterSetParam "ColladaSingleMatrix" false
	fbxExporterSetParam "ColladaFrameRate" 30.0
	fbxExporterSetParam "Convert2Tiff" false
	fbxExporterSetParam "ConvertUnit" "cm"
	fbxExporterSetParam "EmbedTextures" false
	fbxExporterSetParam "FileVersion" (getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Version")
	fbxExporterSetParam "FilterKeyReducer" false
	fbxExporterSetParam "GeomAsBone" true
	fbxExporterSetParam "GenerateLog" false
	fbxExporterSetParam "Lights" false
	--fbxExporterSetParam "LoadExportPresetFile"
	fbxExporterSetParam "NormalsPerPoly" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "NormalsPerPoly"))
	fbxExporterSetParam "PointCache" false
	--fbxExporterSetParam "PopSettings"
	fbxExporterSetParam "PreserveInstances" false
	--fbxExporterSetParam "PushSettings"
	fbxExporterSetParam "RemoveSingleKeys" false
	fbxExporterSetParam "Resampling" 30.0
	--fbxExporterSetParam "ResetExport"
	fbxExporterSetParam "ScaleFactor" 1.0
	--fbxExporterSetParam "SelectionSet"
	fbxExporterSetParam "SelectionSetExport" false
	fbxExporterSetParam "Shape" true --// Export the Morpher modifier "blendshapes" -- same as the Morph option in FBX options > Deformations
	fbxExporterSetParam "Skin" true
	fbxExporterSetParam "ShowWarnings" true
	fbxExporterSetParam "SmoothingGroups" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "SmoothingGroups"))
	fbxExporterSetParam "SmoothMeshExport" false
	--fbxExporterSetParam "SplitAnimationIntoTakes"
	fbxExporterSetParam "TangentSpaceExport" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "TangentSpaceExport"))
	fbxExporterSetParam "Triangulate" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Triangulate"))
	fbxExporterSetParam "UpAxis" "Y"
	fbxExporterSetParam "UseSceneName" false
	fbxExporterSetParam "PreserveEdgeOrientation" (execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation"))
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_findSkinModifier obj =
(
	local skinFound = false
	for i = 1 to obj.modifiers.count do
	(
		if classof obj.modifiers[i] == Skin do
		(
			skinFound = true
			exit
		)
	)
	skinFound
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectSkinBones obj noMesh:true selectNodes:true =
(
	--// noMesh:false ... will select the skin mesh as well as the bones
	--// Assumes a skin modifier exists in the modifier stack and the object is selected
	
	try --// Grouped objects will cause a crash, hence the Try-Catch
	(
		select obj
		setCommandPanelTaskMode #modify
		modPanel.setCurrentObject obj.modifiers[#Skin]
		
		local boneListCount = skinOps.GetNumberBones obj.skin
		if boneListCount > 0 do
		(
			local a_nodes = #()
			for i = 1 to boneListCount do
			(
				try(append a_nodes (getNodeByName (skinOps.getBoneName obj.skin i 1)))catch() --// Try Catch needed in case the index is invalid
			)
			if noMesh do
			(
				clearSelection()
			)
			
			if selectNodes then
			(
				selectmore a_nodes
			)
			else --// Return the nodes instead as a collection
			(
				a_nodes
			)
		)
	)
	catch
	(
		messageBox "Error: Skin bones selection failed.\n" title:""
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectHierarchyRoot obj = --// Recursive function
(
	if selection.count > 0 do
	(
		if obj.parent != undefined do
		(
			select obj.parent
			f_selectHierarchyRoot selection[1]
		)
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectHierarchy obj =
(
	if selection.count > 0 do
	(
		g_LBTools_FBX.f_selectHierarchyRoot obj
		select (g_LBTools_FBX.f_getAllChildren selection[1] a_nodes:(selection as array))
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_makeSubfolder str_file str_sourcePath str_outputPath addBackslash:false =
(
	local str_filePath = (getFileNamePath str_file)
	local str_subFolder = ""
	if addBackslash then
	(
		str_subFolder = subString str_filePath (str_sourcePath.count + 1) str_filePath.count
	)
	else
	(
		str_subFolder = subString str_filePath (str_sourcePath.count + 2) str_filePath.count
	)
	local str_subFolderPath = (str_outputPath + str_subFolder)
	makeDir str_subFolderPath
	str_subFolderPath
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_getFilesRecursive str_rootPath str_pattern includeRoot:true =
(
	--// This function was taken from MXS help and modified slightly
	
	a_dir = getDirectories (str_rootPath + @"\*")
	for d in a_dir do
	(
		join a_dir (getDirectories (d + @"\*"))
	)
	local a_files = #()
	if includeRoot do
	(
		join a_files (getFiles (str_rootPath + @"\" + str_pattern))
	)
	for f in a_dir do
	(
		join a_files (getFiles (f + str_pattern))
	)
	return a_files
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_getMaxFileNames a_maxFiles str_rootPath:undefined removeMaxSuffix:false = --// Pass in an array of maxfiles and their full paths using the getFiles method
(
	local a_names = #()
	
	if str_rootPath != undefined then --// If str_rootPath is provided, it includes the subfolder structure under the root path
	(
		for f in a_maxFiles do
		(
			
			local str_subPath = subString f (str_rootPath.count + 2) f.count
			if removeMaxSuffix then
			(
				append a_names (replace str_subPath (str_subPath.count-3) 4 "") --// Removes the .max suffix
			)
			else
			(
				append a_names str_subPath --// Keeps the .max suffix
			)
		)		
	)
	else
	(
		for f in a_maxFiles do
		(
			append a_names (getFilenameFile f)
		)
	)
	a_names
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_sortMaxfilesListByDate a_maxFiles = --// Pass in an array of maxfiles and their full paths using the getFiles method
(	
	fn f_reformatDateTime d =
	(		
		local a_splitFull = filterstring d " "
		local a_splitDate = filterstring a_splitFull[1] "/"
		local a_splitTime = filterstring a_splitFull[2] ":"
		
		--// DATE REFORMAT
		if a_splitDate[1].count == 1 do (a_splitDate[1] = "0" + a_splitDate[1])
		if a_splitDate[2].count == 1 do (a_splitDate[2] = "0" + a_splitDate[2])
		if a_splitDate[3].count == 1 do (a_splitDate[3] = "0" + a_splitDate[3])
		local str_date = a_splitDate[3] + "/" + a_splitDate[1] + "/" + a_splitDate[2] --// Date formatting... [year/month/day]
		
		--// TIME REFORMAT
		if a_splitFull[3] == "PM" AND a_splitTime[1] != "12" then --// convert the hour part (1-11) to 24 hour format
		(
			a_splitTime[1] = (execute (a_splitTime[1]) + 12) as string
		)
		else --// AM so just pad with a zero at the start
		(
			if a_splitTime[1].count == 1 do (a_splitTime[1] = "0" + a_splitTime[1])
		)
		if a_splitTime[2].count == 1 do (a_splitTime[2] = "0" + a_splitTime[2])
		if a_splitTime[3].count == 1 do (a_splitTime[3] = "0" + a_splitTime[3])
		local str_time = a_splitTime[1] + ":" + a_splitTime[2] + ":" + a_splitTime[3]
		
		return (str_date + " " + a_splitFull[3] + " " + str_time)
	)
	
	--// Convert the full maxfile path into a format suitable for date sorting
	local a_sort = #()
	for f in a_maxFiles while not keyboard.escPressed do
	(
		local str_newDateFormat = f_reformatDateTime (getFileModDate f)
		append a_sort (str_newDateFormat + "#" + (getFileNameFile f) + "#" + (getFileNamePath f))
	)
	sort a_sort
	--// Invert the date sort so the last date modified is first in the list
	a_sort = for i = a_sort.count to 1 by -1 collect a_sort[i]

	--// Convert the date sort array to a new array of full maxfile path strings
	local a_sorted = #()
	for str in a_sort while not keyboard.escPressed do
	(
		
		local str_fileName = (filterString str "#")[2] + ".max"
		local str_filePath = (filterString str "#")[3]
		append a_sorted (str_filePath + str_fileName)
	)
	
	a_sorted
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_linearAttach a_objs =
(
	convertToPoly a_objs[1]
	undo off
	(
		for obj in 2 to a_objs.count do
		(
			polyop.attach a_objs[1] a_objs[obj]
		)
	)
	a_objs[1]
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_combineArrayIntoString a =
(
	local s = ""
	for i = 1 to (a.count - 1) do --// Ignore the last array item
	(
		if a[i] == "" then
		(
			append s "_"
		)
		else
		(
			append s (a[i] + "_")
		)
	)
	substring s 1 (s.count - 1)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_getAllChildren obj &a_nodes:#() = --// Recursive function
(
	for c in obj.children do
	(
		append a_nodes c
		f_getAllChildren c a_nodes:a_nodes
	)
	a_nodes
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

--// Removes the Level of Detail association and explodes the group, returns the previously grouped objects (LOD's)
fn f_removeLOD groupHead =
(
	local a_objs = for obj in geometry where obj.parent == groupHead collect obj
	explodeGroup groupHead
	for obj in a_objs where classOf (getVisController(obj)) == LOD_Controller do
	(
		obj.visibility = bezier_float()
	)
	a_objs
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_createFBXexportINI =
(
	--// Setup the default settings on first time creation of the INI file
	if NOT (doesFileExist g_LBTools_FBX.FBXexportINI) do
	(
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "SmoothingGroups" "true"
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "false" --// Split Per-Vertex Normals --// NOTE: If this is TRUE, Unreal will display a warning about smoothing groups when importing
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "false" --// Tangents and Binormals
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Triangulate" "true"
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "true"
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Version" "FBX202000"
		setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "ASCII" "false"
	)
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_FBXexportSettingsUI =
(
	rollout FBXsettings_rollout "LB Tools"
	(
		local a_fbxVersion = #("#FBX202000","#FBX201900","FBX201800")
		
		label lbl_title "====  Export Tool Settings  ===="
		label lbl_gap1 ""
		group "Root Path"
		(
			label lbl_text1 "Add a Root Path which will be used to shorten the text file based dropdown list of Export Paths." align:#left
			label lbl_text2	 ">> Example: C:\Program Files  {no backslash}" align:#left
			label lbl_gap2 ""
			label lbl_text3 "Click the ''C'' button to check if the string entered is a valid directory to save as the Root Path." align:#left
			label lbl_gap3 ""
			edittext edtxt_rootPath "Root Path:" width:450 across:2 align:#left
			button btn_checkRootPath "C" align:#right offset:[5,0] tooltip:"Checks to see if the string entered is a valid directory to save as the Root Path"
		)
		group "FBX Export Settings"
		(
			label lbl_1 "Geometry:" align:#left
			checkbox chkbx_smoothGroups "Smoothing Groups" offset:[20,0]
			checkbox chkbx_splitPerVertex "Split Per-Vertex Normals" offset:[20,0]
			checkbox chkbx_tangentBinormals "Tangent and Binormals" offset:[20,0]
			checkbox chkbx_triangulate "Triangulate" offset:[20,0]
			checkbox chkbx_preserveEdges "Preserver Edge Orientation" offset:[20,0]
			label lbl_2 "File Version:" align:#left
			dropdownlist ddl_fbxVersion "" items:#("FBX 2020","FBX 2019","FBX 2018") width:120 across:2 offset:[20,0] tooltip:"Choose which FBX version to output as"
			checkbox chkbx_fbxASCII "ASCII output (OFF for binary)" checked:false align:#left offset:[-90,3] tooltip:"Output the FBX file in ASCII format for easy reading, otherwise output as binary"
		)
		group "Miscellaneous"
		(
			button btn_removeLOD "Remove Level of Detail (LoD) on selection" align:#left tooltip:"Explodes any selected LoD groups and removes the LoD associations with the plugin (i.e. makes all LOD's visible)"
			label lbl_gap ""
			label lbl_3 "Title decal theme:" across:2 align:#left
			radiobuttons rdobtns_titleTheme "" labels:#("Dark","Light") align:#left offset:[-140,0] offsets:#([-5,0],[5,0])
		)
		
		on FBXsettings_rollout open do
		(
			local str_rootPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
			local a_rootPath = getDirectories (str_rootPath)
			if a_rootPath.count > 0 AND str_rootPath != "*** BLANK ***" then
			(
				edtxt_rootPath.text = str_rootPath
			)
			else
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath" @"*** BLANK ***"
				edtxt_rootPath.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
			)
			
			rdobtns_titleTheme.state = execute(getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme")
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "SmoothingGroups") then
			(
				chkbx_smoothGroups.checked = true
			)
			else
			(
				chkbx_smoothGroups.checked = false
			)
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "NormalsPerPoly") then
			(
				chkbx_splitPerVertex.checked = true
			)
			else
			(
				chkbx_splitPerVertex.checked = false
			)
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "TangentSpaceExport") then
			(
				chkbx_tangentBinormals.checked = true
			)
			else
			(
				chkbx_tangentBinormals.checked = false
			)
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Triangulate") then
			(
				chkbx_triangulate.checked = true
			)
			else
			(
				chkbx_triangulate.checked = false
			)
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation") then
			(
				chkbx_preserveEdges.checked = true
			)
			else
			(
				chkbx_preserveEdges.checked = false
			)
			
			local str_fbxVersion = getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Version"
			for index = 1 to a_fbxVersion.count do
			(
				if (findString a_fbxVersion[index] str_fbxVersion) == 1 do
				(
					ddl_fbxVersion.selection = index
					exit
				)
			)
			
			if execute(getIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "ASCII") then
			(
				chkbx_fbxASCII.checked = true
			)
			else
			(
				chkbx_fbxASCII.checked = false
			)
		)
		
		on FBXsettings_rollout close do
		(
			if (getDirectories edtxt_rootPath.text).count > 0 then
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath" edtxt_rootPath.text
				g_LBTools_FBX.f_openExporterUI()
			)
			else
			(
				messageBox "Error: Invalid Root Path!" title:""
				g_LBTools_FBX.f_openExporterUI()
			)
		)
		
		on btn_removeLOD pressed do
		(
			for obj in selection where isGroupMember obj AND classOf (getVisController(obj)) == LOD_Controller do
			(
				g_LBTools_FBX.f_removeLOD obj.parent
			)
		)
		
		on rdobtns_titleTheme changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme" (theState as string)
		)
		
		on btn_checkRootPath pressed do
		(
			if (getDirectories edtxt_rootPath.text).count > 0 then
			(
				messageBox "Directory is valid. Root path updated and saved." title:""
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath" edtxt_rootPath.text
			)
			else
			(
				messageBox "Error: Invalid Root Path!" title:""
				edtxt_rootPath.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
			)
		)
		
		on chkbx_smoothGroups changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "SmoothingGroups" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "SmoothingGroups" "false"
			)
		)
		
		on chkbx_splitPerVertex changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "NormalsPerPoly" "false"
			)
		)
		
		on chkbx_tangentBinormals changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "TangentSpaceExport" "false"
			)
		)
		
		on chkbx_triangulate changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Triangulate" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Triangulate" "false"
			)
		)
		
		on chkbx_preserveEdges changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "PreserveEdgeOrientation" "false"
			)
		)
		
		on ddl_fbxVersion selected val do
		(
			setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "Version" a_fbxVersion[val]
		)
		
		on chkbx_fbxASCII changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "ASCII" "true"
			)
			else
			(
				setIniSetting g_LBTools_FBX.FBXexportINI "FBX Export Settings" "ASCII" "false"
			)
		)
	) --// end rollout
	
	fn f_openExportSettingsUI =
	(
		try(destroyDialog g_LBTools_FBX.FBXsettings_rollout)catch()
		createDialog g_LBTools_FBX.FBXsettings_rollout width:500
	)
	
	g_LBTools_FBX.f_openExportSettingsUI()
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_useSameFolder =
(
	local str_sameFolder = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder"
	local okToUse = true
	
	if execute (str_sameFolder) do
	(
		if maxFileName == "" then
		(
			messageBox "Error: ''Same folder as scene file'' is checked and no scene file is currently open." title:""
			okToUse = false
		)
		else
		(
			if (queryBox "WARNING:\n\nYou are about to export to the SAME FOLDER as the scene file.\n\nOK to continue?\n\nOtherwise, disable the checkbox: Same folder as scene file." title:"") == false do
			(
				okToUse = false
			)
		)
	)
	okToUse
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_getBatchAnimExportPath =
(
	local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
	local str_sameFolder = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder"
	local exportPathOK = false

	if execute (str_sameFolder) do --// Use same folder was checked and a scene file is open
	(
		str_exportPath = substring maxFilePath 1 (maxFilePath.count - 1)
		exportPathOK = true
	)

	if (getDirectories str_exportPath).count > 0 then --// Check the exstiing Export Path string to see if it's valid
	(
		g_LBTools_FBX.str_batchOutputPath = str_exportPath + @"\"
		exportPathOK = true
	)
	else
	(
		g_LBTools_FBX.str_batchOutputPath = ""
	)
	exportPathOK
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_batchExportAnimations a_processFiles =
(
	local str_selectedMaxfiles = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles"
	local str_recurseSubfolders = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders"
	local str_ignoreTimeTags = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags"
	
	--// Listener output
	format "==========================================================\n// Batch Animation Summary - %\n----------------------------------------------------------\n" localTime
	format "Source path... %\n----------------------------------------------------------\n" (g_LBTools_FBX.str_batchSourcePath + @"\")
	
	--// Record the currently open scene for later reopening after the batch processing is complete
	local str_openedFile = (maxfilepath + maxfilename)
	
	--// Export the animation from each maxfile
	for f in a_processFiles while not keyboard.escPressed do
	(
		loadMaxFile f quiet:true
		
		--// Listener output
		format "Scene file... %\n" f
		
		if maxFileName != "" then --// In case the maxfile can't open due to Max version mismatch
		(
			unhide objects doLayer:true
			unfreeze objects doLayer:true
			
			--// Open all groups in case there are LoD's or other non-LoD groups. The individual meshes must be accessible (opened group) in order to find the export skeleton bones
			for obj in objects where (isGroupMember obj) do
			(
				setGroupOpen obj.parent true 
			)
			
			local objFound = false
			local str_skinNameBatch = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch"
			if str_skinNameBatch != "*** BLANK ***" AND str_skinNameBatch != "" then --// Use a specified node (mesh name) with a Skin modifier, in case there's multiple skinned meshes in the same scene.
			(
				local meshNameNode = getNodeByName str_skinNameBatch
				if meshNameNode != undefined AND (g_LBTools_FBX.f_findSkinModifier meshNameNode) do
				(
					objFound = true
					g_LBTools_FBX.f_selectSkinBones meshNameNode
					g_LBTools_FBX.f_selectHierarchy selection[1]
				)
			)
			else --// Attempts to select a valid skin mesh and export rig. It uses the first skin mesh found and ignores the rest. Not as comprehensive searching as the single scene export method.
			(
				for obj in geometry where (g_LBTools_FBX.f_findSkinModifier obj) do
				(
					g_LBTools_FBX.f_selectSkinBones obj
					g_LBTools_FBX.f_selectHierarchy selection[1]
					objFound = true
					exit
				)
			)
			
			if objFound then
			(
				--// Setup the output path depending on the state of the Recurse subfolder option in the UI
				local str_outputPath = ""
				if execute (str_recurseSubfolders) then --// Traverse all maxfiles in subfolders under the picked root path
				(
					--format "GetFileNamePath:  %\n" (getFileNamePath f)
					if (getFileNamePath f) == (g_LBTools_FBX.str_batchSourcePath + @"\") then --// Maxfile should be exported directly to the Export Path
					(
						--format "Don't create subfolders.\n"
						str_outputPath = g_LBTools_FBX.str_batchOutputPath
					)
					else --// Maxfile is in a subfolder structure that needs creating under the Export Path
					(
						str_outputPath = (g_LBTools_FBX.f_makeSubfolder f g_LBTools_FBX.str_batchSourcePath g_LBTools_FBX.str_batchOutputPath) --// Force create and return the output root path plus the subfolder
						--format "Subfolder structure created:  %\n" str_outputPath
					)
				)
				else --// Single path only
				(
					str_outputPath = g_LBTools_FBX.str_batchOutputPath
				)

				sliderTime += 1 --// This is to update any animation CA's or controllers
				
				--// Check if there are any time tags in the scene for composite animations
				local ttCount = FrameTagManager.GetTagCount()
				if ttCount > 0 AND (execute (str_ignoreTimeTags) == false) then
				(
					try --// Adding a Try Catch to prevent crashes when time tags are not valid
					(
						format "Animation (time tags) export...\n"
						local aaRange = animationRange
						for i = 1 to ttCount do
						(
							--//******************************************************************************************************************************************************
							--// NOTE:
							--// The FBX exporter will respect all keys on what you have selected during the export. This means you can't have keys existing outside of the time range as it will
							--// inlcude those keys as well. Instead you need to have your export rig indirectly controlled using constraints with no keys then time ranges will be respected. This
							--// is neccessary for the time tags method of exporting.
							--//******************************************************************************************************************************************************
							
							--// Read the time tag to extract the name and range specified
							local str_tt = FrameTagManager.GetNameByID i
							local a_tt = (filterString str_tt " ")
							local str_tagName = a_tt[1]
							local a_range = (filterString a_tt[2] "-")
							animationRange = interval (a_range[1] as time) (a_range[2] as time)
							
							g_LBTools_FBX.f_initializeFBXexport()
							
							--// Export the file and handle read only
							local str_exportFileName = (str_outputPath + str_tagName + ".FBX")
							if doesFileExist str_exportFileName then
							(
								if getFileAttribute str_exportFileName #readonly == false then
								(
									exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP
									format "File exported ok... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
								)
								else
								(
									setFileAttribute str_exportFileName #readonly false
									exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP
									format "File was readonly and overwritten... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
								)
							)
							else --// First time creation so just export it
							(
								exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP
								format "File exported ok... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
							)
						)
						animationRange = aaRange
						sliderTime = 0
					)
					catch
					(
						format "Error: Scene >> % << may have invalid time tags.\n" maxFileName
					)
				)
				else --// No time tags found
				(
					g_LBTools_FBX.f_initializeFBXexport()
					
					--// Listener output
					format "Animation export...\n"
					
					--// Export the file and handle read only
					local str_exportFileName = (str_outputPath + (getFilenameFile maxFileName) + ".FBX")
					if doesFileExist str_exportFileName then --// First check to make sure the output FBX file is writable
					(
						if getFileAttribute str_exportFileName #readonly == false then
						(
							exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP
							format "File exported ok... % ... Range:%-%(%)\n" ((getFilenameFile maxFileName) + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
						)
						else
						(
							format "Error: File is readonly!... %\n" ((getFilenameFile maxFileName) + ".FBX")
						)
					)
					else --// First time creation so just export it
					(
						exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP
						format "File exported ok... % ... Range:%-%(%)\n" ((getFilenameFile maxFileName) + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
					)
				)
				local str_sceneFile = (maxfilepath + maxfilename)
				--(str_outputPath + (replace (getFileNamePath (subString str_sceneFile (str_outputPath.count + 1) str_sceneFile.count)) 1 1 ""))
				format "Export Path... %\n----------------------------------------------------------\n" str_outputPath
			)
			else
			(
				format "Error: Skin mesh name >>> % <<< not found in maxfile... %\n" str_skinNameBatch ((getFilenameFile maxFileName) + ".max")
			)
		)
		else
		(
			format "Error: Maxfile >>> % <<< could not open, possibly due to version mismatch.\n" (filenameFromPath f)
		)
	)
	format "// Batch Animation Completed - %\n==========================================================\n" localTime
	messageBox "Batch export animations completed.\n\nCheck the Maxscript listener for details." title:""
	
	if str_openedFile != "" AND doesFileExist str_openedFile then
	(
		loadMaxFile str_openedFile quiet:true
	)
	else
	(
		resetMaxFile #noPrompt
	)
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectMaxfilesUI =
(
	rollout maxFileSelection_rollout "LB Tools"
	(
		local imageFile1 = (getDir #usericons) + "\LBTools_Folder_Image.bmp"
		
		label lbl_title "====  Batch Animation Exporting (selection method)  ===="
		editText edtxt_sourcePath "Source Path:" width:545 readOnly:true across:2
		button btn_browsePath "[...]" width:26 align:#right
		label lbl_txt1 "Select maxfiles to export skeletal animation from..." align:#left offset:[5,10] across:2
		button btn_folderSourcePath "" align:#right offset:[-2,0] images:#(imageFile1, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window showing the Source Path."
		button btn_sortByName "Sort By Name" width:285 height:18 across:2
		button btn_sortByDate "Sort By Date" width:285 height:18
		multiListBox mlstbx_maxfiles "" align:#left selection:#{} height:30
		group ""
		(
			button btn_batchExport ">>  Export Animation From Selection  <<" width:550 height:40
		)

		fn	f_getFilesFromSource =
		(
			local str_recurseSubfolders = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders"
			if execute (str_recurseSubfolders) then
			(
				g_LBTools_FBX.a_maxfilesToBatch = g_LBTools_FBX.f_getFilesRecursive g_LBTools_FBX.str_batchSourcePath "*.max"
			)
			else
			(
				g_LBTools_FBX.a_maxfilesToBatch = getFiles (g_LBTools_FBX.str_batchSourcePath + "\*.max")
			)
		)
		
		on maxFileSelection_rollout open do
		(
			edtxt_sourcePath.text = g_LBTools_FBX.str_batchSourcePath + @"\"
			f_getFilesFromSource()
			mlstbx_maxfiles.items = sort (g_LBTools_FBX.f_getMaxFileNames g_LBTools_FBX.a_maxfilesToBatch str_rootPath:g_LBTools_FBX.str_batchSourcePath)
		)
		
		on btn_folderSourcePath pressed do
		(
			local str_sourcePath = replace edtxt_sourcePath.text edtxt_sourcePath.text.count 1 ""
			format "str_sourcePath:  %\n" str_sourcePath
			if str_sourcePath != "" AND (getDirectories str_sourcePath).count > 0 then
			(
				ShellLaunch "explorer.exe" str_sourcePath
			)
			else
			(
				messageBox "Error: Source Path specified is invalid." title:""
			)
		)
		
		on btn_browsePath pressed do
		(
			local str_path = getSavePath caption:"Select a folder of animation maxfiles to export..." initialDir:g_LBTools_FBX.str_batchSourcePath
			if  str_path != undefined then
			(
				g_LBTools_FBX.str_batchSourcePath = str_path
				g_LBTools_FBX.f_openSelectMaxfilesUI()
			)
			else
			(
				messageBox "Error: Export Path not valid." title:""
			)
		)
		
		on btn_sortByName pressed do
		(
			mlstbx_maxfiles.items = sort (g_LBTools_FBX.f_getMaxFileNames g_LBTools_FBX.a_maxfilesToBatch str_rootPath:g_LBTools_FBX.str_batchSourcePath)
			mlstbx_maxfiles.selection = 0
		)
		
		on btn_sortByDate pressed do
		(
			local a_dateSorted = g_LBTools_FBX.f_sortMaxfilesListByDate g_LBTools_FBX.a_maxfilesToBatch
			mlstbx_maxfiles.items  = g_LBTools_FBX.f_getMaxFileNames a_dateSorted str_rootPath:g_LBTools_FBX.str_batchSourcePath
			mlstbx_maxfiles.selection = 0
		)
		
		on btn_batchExport pressed do
		(
			if g_LBTools_FBX.f_useSameFolder() do
			(
				--// Get the export path and check if it's valid	
				if g_LBTools_FBX.f_getBatchAnimExportPath() == false then
				(
					messageBox "Error: Export Path not valid." title:""
				)
				else
				(
					if mlstbx_maxfiles.selection != #{} do
					(
						local a_filesToProcess = #()
						for n in (mlstbx_maxfiles.selection as array) do
						(
							append a_filesToProcess (g_LBTools_FBX.str_batchSourcePath + @"\" + mlstbx_maxfiles.items[n])
						)
						
						if a_filesToProcess.count > 0 then
						(
							g_LBTools_FBX.f_batchExportAnimations a_filesToProcess
						)
						else
						(
							messageBox "Error: Select one or more maxfiles to process." title:""
						)
					)
				)
			)
		)
	) --// end rollout
	
	fn f_openSelectMaxfilesUI =
	(
		try(destroyDialog g_LBTools_FBX.maxFileSelection_rollout)catch()
		createDialog g_LBTools_FBX.maxFileSelection_rollout width:600 height:575
	)
	
	g_LBTools_FBX.f_openSelectMaxfilesUI()
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_exportFBX_Static =
(
	--// Retrieve the INI settings (assumes the INI file exists and has initial settings)
	local str_displayLog = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog"
	local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
	local str_sameFolder = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder"
	local str_singleMesh = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh"
	local str_centreAtOrigin = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin"	
	local str_orientWorld = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld"
	local str_centrePivot = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot"
	local str_singleFBX = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX"
	
	local str_fileName = ""
	local str_exportFileName = ""
	local str_oldExports = ""
	local str_newExports = ""
	local str_roFiles = ""
	local str_outputPath = ""
	local a_exportObjsSingleFBX = #()

	fn f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles =
	(
		if doesFileExist str_exportFileName then
		(
			if getFileAttribute str_exportFileName #readonly == false then
			(
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
				append str_oldExports (str_fileName + "\n\t")
			)
			else
			(
				setFileAttribute str_exportFileName #readonly false
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 	
				append str_oldExports (str_fileName + "\n\t")
				append str_roFiles (str_fileName + "\n\t")
			)
		)
		else
		(
			exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
			append str_newExports (str_fileName + "\n\t")
		)
	)

	fn f_logFileOutput str =
	(
		local txtFile = @"C:\Temp\LBTools_ExportLog.txt"
		deleteFile txtFile
		local edit_txtFile = openFile txtFile mode:"w"
		format "LB Tools - Export FBX to Unreal - Static Meshes - %%" localTime "\n\n" to:edit_txtFile
		format "%\n" str to:edit_txtFile
		close edit_txtFile
		return txtFile
	)
	
	--// Reset the log file string contents
	g_LBTools_FBX.str_outputToLog = ""
	
	if execute (str_sameFolder) then
	(
		--// Output to the current maxfile scene path
		if maxFilePath != "" then
		(
			str_outputPath = substring maxFilePath 1 (maxFilePath.count - 1)
		)
		else
		(
			str_outputPath = getDir #scene
		)
	)
	else --// Read the ini file and extract the output path from that
	(
		str_outputPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
	)

	if (getDirectories str_outputPath).count > 0 then --// It's a valid output path, so continue
	(
		local okToContinue = true
		
		if g_LBTools_FBX.f_useSameFolder() then
		(
			if selection.count == 0 AND g_LBTools_FBX.exportFBX_Rollout.chkbx_useStaticSS.checked == false do
			(
				if (queryBox "WARNING:\n\nYou have nothing selected. The exporter will attempt to export all static meshes and collisions found.\n\nOK to continue?" title:"") == false do
				(
					okToContinue = false
				)
			)
		)
		else
		(
			okToContinue = false
		)
		
		if okToContinue do
		(
			--// Listener output
			format "==========================================================\n// Static Export Summary - %\n----------------------------------------------------------\n" localTime
			if execute (str_singleMesh) == false do
			(
				format ">> Check the log file for details.\n"
			)
			format "Scene file... %\n" (maxFilePath + maxFileName)
			
			str_outputPath = str_outputPath + @"\"
			
			local a_separateObjs = #() --// Separate objects (i.e. not part of LOD groups)
			local a_lodHeads = #() --// LOD object headers that use the Level of Detail utility
			local a_allObjs = #() --// Combined separate objects with LoD headers
			
			--// Check if the selection sets option is on and there are selection sets in the list
			local flag_useSelectionSets = false
			if g_LBTools_FBX.exportFBX_Rollout.chkbx_useStaticSS.checked AND g_LBTools_FBX.exportFBX_Rollout.ddl_SS.items.count > 0 do
			(
				flag_useSelectionSets = true
			)
			
			fn f_collectSSnodes setIndex = --// Collect and return the selection set nodes
			(
				local a_SSnodes = #()
				local itemsCount = getNamedSelSetItemCount setIndex
				for i = 1 to itemsCount do
				(
					append a_SSnodes (getNamedSelSetItem setIndex i)
				)
				a_SSnodes
			)
			
			fn f_collectObjects obj a_lodHeads a_separateObjs = --// Collect and store all of the separate objects and LoD heads into two different arrays
			(
				objParent = obj.parent
				if objParent != undefined AND isGroupHead objParent then --// The object is part of a Level of Detail (LoD) utiility group set (i.e. it's parent Dummy is a group header)
				(
					lodFound = false
					for lodHead in a_lodHeads do --// Does the LoD head already exist in the array
					(
						if objParent == lodHead do
						(
							lodFound = true
							break
						)
					)
					if NOT lodFound do --// Found a unique LoD to add
					(
						append a_lodHeads objParent
					)
				)
				else --// Not a LoD object
				(
					if (findString obj.name "UCX_") == undefined AND (findString obj.name "UBX_") == undefined AND (findString obj.name "UCP_") == undefined AND (findString obj.name "USP_") == undefined do
					(
						--// Found a non-collision object - i.e. a static mesh
						append a_separateObjs obj
					)
				)
			)
			
			if flag_useSelectionSets then --// Collect all geometry in the selection set, excluding skin meshes and bone geometry
			(
				local a_ssObjs = f_collectSSnodes g_LBTools_FBX.exportFBX_Rollout.ddl_SS.selection
				for obj in a_ssObjs where (superClassOf obj == geometryClass) AND (classof obj.modifiers[1] != Skin) AND (classof obj != BoneGeometry) do
				(
					f_collectObjects obj a_lodHeads a_separateObjs
				)
			)
			else --// Not using selection sets
			(
				local a_selObjs = getCurrentSelection()
				if a_selObjs.count > 0 then  --// Collect all geometry in the selection, excluding skin meshes and bone geometry
				(
					for obj in a_selObjs where (superClassOf obj == geometryClass) AND (classof obj.modifiers[1] != Skin) AND (classof obj != BoneGeometry) do
					(
						f_collectObjects obj a_lodHeads a_separateObjs
					)
				)
				else --// Collect all geometry in the scene, excluding skin meshes and bone geometry
				(
					for obj in geometry where (superClassOf obj == geometryClass) AND (classof obj.modifiers[1] != Skin) AND (classof obj != BoneGeometry) do
					(
						f_collectObjects obj a_lodHeads a_separateObjs
					)
				)
			)
			
			--// Explode all separate object groups and effectively treat them as a selection set
			explodeGroup a_separateObjs
			--// Unlink all separate object and their collision volumes so correct collision linkage can occur later
			for obj in a_separateObjs do
			(
				obj.parent = undefined
				--// Add them to the all objects array to combine with the LoD header objects
				append a_allObjs obj
			)
			
			--// Add the LoD headers to the same array as the separate objects (now that they've been exploded and unlinked)
			for lodHead in a_lodHeads do
			(
				append a_allObjs lodHead
			)
				
			--// Export all meshes
			if a_allObjs.count > 0 then
			(
				setCommandPanelTaskMode #create
				with redraw off
				(
					if execute (str_singleMesh) then --// SINGLE MESH and a single FBX file (preserves the separate meshes in the scene by making duplicates)
					(
						local a_objsToCopy = #()
						for obj in a_allObjs do
						(
							if isGroupHead obj then 
							(
								--// Find all LoD's and explode them
								local a_objs = g_LBTools_FBX.f_removeLOD obj
								for o in a_objs do
								(
									append a_objsToCopy o
								)
							)
							else
							(
								append a_objsToCopy obj
							)
						)

						--// Make copies of all the geometry objects
						local a_copies = #()
						for obj in a_objsToCopy do
						(
							if superclassof obj == geometryClass do
							(
								local copyObj = copy obj
								append a_copies copyObj
							)
						)
						
						--// Attach the copied objects and name the output mesh the same as the maxfile name
						select a_copies
						local copyObj = g_LBTools_FBX.f_linearAttach a_copies
						copyObj.name = getFilenameFile maxfilename
						
						--// Centre the mesh at the origin before adjusting the pivot (for scenes with offset objects)
						if execute (str_centreAtOrigin) do
						(
							if execute (str_orientWorld) do
							(
								copyObj.rotation.x_rotation = 0
								copyObj.rotation.y_rotation = 0
								copyObj.rotation.z_rotation = 0
							)
							if execute (str_centrePivot) do
							(
								copyObj.pivot = copyObj.center
							)
							--copyObj.pivot.z = copyObj.min.z
							copyObj.position = [0,0,0]
						)
						
						--// Export the file
						str_exportFileName = (str_outputPath + (getFilenameFile maxfilename) + ".FBX")
						str_fileName = ((getFilenameFile str_exportFileName) + ".FBX")
						f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
						
						delete copyObj
					)
					else --// SEPARATE MESHES (including LoD's) and FBX files
					(
						append g_LBTools_FBX.str_outputToLog ("Scene file... " + (maxFilePath + maxFileName) + "\n")
						if execute (str_singleFBX) then
						(
							append g_LBTools_FBX.str_outputToLog ("Exported as a single FBX file... " + (@"StaticMeshes_" + (getFilenameFile maxfilename) + ".FBX\n"))
						)
						else
						(
							append g_LBTools_FBX.str_outputToLog ("Exported as separate FBX files using the static mesh names.\n")
						)
						append g_LBTools_FBX.str_outputToLog ("Export Path... " + str_outputPath + "\n\n")
						for obj in a_allObjs do --// Operate on all static meshes and LoD headers stored in the array
						(
							local a_exportObjs = #() --// This array holds the static meshes, LoD headers and their collision volumes (if found)
							--format "Static mesh...%\n" obj.name
							--// Add the static mesh for exporting
							append a_exportObjs obj
							--// Collect for exporting outside the loop for single FBX file output
							append a_exportObjsSingleFBX obj
							--// Report the static mesh to the log file
							append g_LBTools_FBX.str_outputToLog ("-------------------------------------------------------------------------------------\n" + "### STATIC MESH...\n" + obj.name)
							if isGroupHead obj do append g_LBTools_FBX.str_outputToLog (" [LoD set]")
							append g_LBTools_FBX.str_outputToLog ("\n\n")
							--// Find all collisions for the static mesh and report them to the log file
							append g_LBTools_FBX.str_outputToLog "### VALID COLLISIONS FOUND...\n"
							
							--// Find any collision volumes for the current object, based on the name and append them to the export array. Except for LoD's - assume they are correct!
							--// Validity of static mesh collision names are based on the Unreal and FBX naming scheme described here... https://docs.unrealengine.com/latest/INT/Engine/Content/FBX/StaticMeshes/index.html
							local a_invalidCollisions = #()
							if NOT isGroupHead obj do
							(
								for o in objects do
								(
									--format "o.name...%\n" o.name
									--// Find a name match with the static mesh and make sure it's a collision object before adding to the export array
									if (findString o.name obj.name) != undefined AND (findString o.name "UCX_") != undefined OR (findString o.name "UBX_") != undefined OR (findString o.name "UCP_") != undefined OR (findString o.name "USP_") != undefined do
									(
	-- 									format "o.name...%\n" o.name
										--// Get rid of the collision prefix
										local str_noPrefix = replace o.name 1 4 ""
										--// Get just the collision name without description to compare it. NOTE: Static mesh names can have underscores, but the collision names must then have an extra underscore at the end (with or without a description).
										local a_cvName = filterString str_noPrefix "_" splitEmptyTokens:true
	-- 									format "a_cvName...%\n" a_cvName
										local str_cvName = ""
										if a_cvName.count > 2 then --// Assume there's underscores in the static mesh name
										(
											--// Return the collision name without the description or underscore at the end
											str_cvName = g_LBTools_FBX.f_combineArrayIntoString a_cvName
										)
										else --// There were no underscores in the collision name, so just compare it with the static mesh name. Also excludes the description.
										(
											str_cvName = a_cvName[1]
										)
--  	 									format "str_cvName...%\n" str_cvName
										--// Compare the collision and static mesh names to see if they match exactly, if so add it to the export array for further naming validation
										if str_cvName == obj.name then
										(
-- 	 										format "%...is a matching collision.\n" o.name
											--// Add the collision object for exporting
											append a_exportObjs o
											--// Collect for exporting outside the loop for single FBX file output
											append a_exportObjsSingleFBX o
											--// Link the valid collision to it's static mesh so that when the mesh is origin centered or world oriented the collisions will follow
											o.parent = obj
											--// Report the found collision object to the log file
											append g_LBTools_FBX.str_outputToLog (o.name + "\n")
										)
										else --// Find any potential invalid collsions for reporting to the log file
										(
											local p = "*" + obj.name + "*"
											if matchpattern o.name pattern:p do
											(
												--format "potentially invalid collision...%\n" o.name
												append a_invalidCollisions o.name
											)
										)
									)
								)
							)

							--// Find collision volumes if the object is part of a LoD set (Note: Does not validate the collisions and report errors like with non-LoD objects - assumes collisions are named correctly)
							if isGroupHead obj do
							(
								select obj --// The dummy that defines the LoD head
								
								--// Find any collision objects linked to the LoD object - in cases where there are multiple LoD's in the scene
								local a_collisionObjs = for child in obj.children where (findString child.name "UCX_") != undefined OR (findString child.name "UBX_") != undefined OR (findString child.name "UCP_") != undefined OR (findString child.name "USP_") != undefined collect child
								if a_collisionObjs.count > 0 then --// Found collision volumes linked to the LoD group (for association with that group compared to other LoD's in the scene)
								(
									for o in a_collisionObjs do
									(
										--// Add the collision object for exporting
										append a_exportObjs o
										--// Collect for exporting outside the loop for single FBX file output
										append a_exportObjsSingleFBX o
										--// Report the found collision object to the log file
										append g_LBTools_FBX.str_outputToLog (o.name + "\n")
									)
								)
								else --// Collision objects were not linked to the LoD group, so search the scene instead (Note: This assumes the scene has only one LoD object with associated collisions)
								(
									--// Get the group objects that are not the group head dummy (i.e. LOD meshes)
	 								local a_lodObjs = for selObj in selection where NOT (isGroupHead selObj) collect selObj
									
									--// Find any collision volumes in the scene that match any of the LOD's within the LoD set
	 								for o in objects do
	 								(
	 									for lodObj in a_lodObjs do
	 									(
	 										if (findString o.name lodObj.name) != undefined AND (findString o.name "UCX_") != undefined OR (findString o.name "UBX_") != undefined OR (findString o.name "UCP_") != undefined OR (findString o.name "USP_") != undefined do
	 										(
	 											--// Add the collision object for exporting
	 											append a_exportObjs o
												--// Collect for exporting outside the loop for single FBX file output
												append a_exportObjsSingleFBX o
	 											--// Link the valid collision to it's LoD set so that when the group is origin centered or world oriented the collisions will follow
	 											o.parent = obj
	 											--// Report the found collision object to the log file
	 											append g_LBTools_FBX.str_outputToLog (o.name + "\n")
	 										)
	 									)
	 								)
								)
							)
							
							--// Report any possible invalid collisions to the log file (Note: Does not report any errors for LoD objects)
							append g_LBTools_FBX.str_outputToLog "\n!!! POTENTIALLY INVALID COLLISIONS...\n"
							for c in a_invalidCollisions do
							(
								append g_LBTools_FBX.str_outputToLog (c + "\n")
							)
							
-- 							format "export...%\n" a_exportObjs

							--// Centre the mesh at the origin and adjust the pivot if checked (used on scenes with offset objects)
							if execute (str_centreAtOrigin) do
							(
								if execute (str_orientWorld) do
								(
									obj.rotation.x_rotation = 0
									obj.rotation.y_rotation = 0
									obj.rotation.z_rotation = 0
								)
								if execute (str_centrePivot) do
								(
									obj.pivot = obj.center
								)
								--obj.pivot.z = obj.min.z
								obj.position = [0,0,0]
							)
							
							--// Unlink all collisions for the object (Note: LoD objects cannot have other objects linked to them that are not part of the LoD set when exporting to FBX - CRASHES!)
							for child in obj.children do
							(
								if (findString child.name "UCX_") != undefined OR (findString child.name "UBX_") != undefined OR (findString child.name "UCP_") != undefined OR (findString child.name "USP_") != undefined do
								(
									child.parent = undefined
								)
							)
							
							if execute (str_singleFBX) == false do --// export a separate FBX file for each static mesh and their collisions
							(
								select a_exportObjs
								
								--// Export the file
								str_fileName = (obj.name + ".FBX")
								str_exportFileName = (str_outputPath + str_fileName)
								f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
							)
						) --// end FOR loop
						
						if execute (str_singleFBX) AND execute (str_singleMesh) == false do --// export all static meshes and their collisions to a single FBX file
						(
							select a_exportObjsSingleFBX
							
							--// Export the file
							str_fileName = @"StaticMeshes_" + (getFilenameFile maxfilename) + ".FBX"
							str_exportFileName = (str_outputPath + str_fileName)
							f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
						)
					)
				)
				
				--// Display export summary
				local str_results = ("=== Export Summary ===\n\n" + "New: \n\t" + str_newExports + "\n" + "Overwritten: \n\t" + str_oldExports + "\n" + "Reconcile: \n\t" + str_roFiles)
				messagebox str_results title:"LB Tools"
				--// Output to log file
				local logFile = f_logFileOutput g_LBTools_FBX.str_outputToLog
				--// Display log file for static mesh exports
				if execute (str_displayLog) AND execute (str_singleMesh) == false do
				(
					ShellLaunch logFile ""
				)
			)
			else
			(
				messageBox "Error: No valid geometry was found to export in either the scene, selection or chosen selection set." title:""
				g_LBTools_FBX.f_openExporterUI()
			)
			
			--// Listener output
			if a_lodHeads.count > 0 do
			(
				format "Level of Detail objects exported...\n"
				for lodHead in a_lodHeads do
				(
					if NOT isDeleted lodHead do
					(
						format "%\n" lodHead.name
					)
				)
			)
			if execute (str_singleMesh) do
			(
				format "Single Mesh export...\n"
				format "File exported ok... %\n" ((getFileNameFile str_exportFileName) + ".FBX")
			)
			if execute (str_singleFBX) AND execute (str_singleMesh) == false do
			(
				format "Single FBX export...\n"
				format "File exported ok... %\n" (@"StaticMeshes_" + (getFilenameFile maxfilename) + ".FBX")
			)
			if execute (str_singleFBX) == false AND execute (str_singleMesh) == false do
			(
				format "Exported as separate FBX files using the static mesh names.\n"
			)
			format "Export Path... %\n----------------------------------------------------------\n" str_outputPath
			format "// Static Export Completed - %\n==========================================================\n" localTime
		)
	)
	else
	(
		messageBox "Error: Export Path not valid." title:""
	)
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectSkinMeshUI a_objs =
(
	rollout selectSkinMesh_Rollout "LB Tools"
	(
		local continuePressed = false
		
		label lbl_error "Multiple hierarchies found with skin meshes and nothing selected."
		label lbl_title "Select a skin mesh to use for exporting bone animation:" align:#left
		listBox lstbx_skinMeshes ""  width:320 height:20 align:#center items:g_LBTools_FBX.a_skinMeshNames selection:0
		button btn_continue ">>  Continue SKELETAL export process  <<" width:320 height:40 align:#center
		
		on selectSkinMesh_Rollout close do
		(
			if continuePressed == false do
			(
				format "Skeletal export process aborted.\n==========================================================\n"
				messageBox "Skeletal export process aborted." title:""
				g_LBTools_FBX.skinMeshSelectProcess = false --// Reset the flag so future exports will fetch the scene
				fetchMaxFile quiet:true
			)
		)
		
		on lstbx_skinMeshes selected index do
		(
			if index > 0 do
			(
				local str_objName = lstbx_skinMeshes.items[index]
				local pickedObj = getNodeByName str_objName
				if isValidNode pickedObj then
				(
					select pickedObj
				)
				else
				(
					messageBox "Error: Skin mesh not found." title:""
					try(destroyDialog selectSkinMesh_Rollout)catch()
					fetchMaxFile quiet:true
				)
			)
		)
		
		on btn_continue pressed do
		(
			if lstbx_skinMeshes.selection > 0 then
			(
				continuePressed = true
				g_LBTools_FBX.skinMeshSelectProcess = false --// Reset the flag so future exports will fetch the scene
				try(destroyDialog selectSkinMesh_Rollout)catch()
				g_LBTools_FBX.f_initializeFBXexport()
				unhide objects doLayer:true
				unfreeze objects doLayer:true
				g_LBTools_FBX.f_exportFBX_Skeletal()
				fetchMaxFile quiet:true
			)
			else
			(
				messageBox "Error: Select a skin mesh from the list or close the dialog to abort." title:""
			)
		)		
	) --// end rollout
	g_LBTools_FBX.a_skinMeshNames = for obj in a_objs collect obj.name
	try(destroyDialog selectSkinMesh_Rollout)catch()
	createDialog selectSkinMesh_Rollout width:340 height:360
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_exportFBX_Skeletal =
(
	--// Retrieve the INI settings (assumes the INI file exists and has initial settings)
	local str_displayLog = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog"
	local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
	local str_sameFolder = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder"
	local str_skeletalMode = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode"
	local str_skinNameGeneral = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral"
	local str_animName = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName"
	local str_ignoreTimeTags = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags"
	
	local str_fileName = ""
	local str_exportFileName = ""
	local str_oldExports = ""
	local str_newExports = ""
	local str_roFiles = ""	
	local str_outputPath = ""
	
	fn f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles =
	(
		if doesFileExist str_exportFileName then
		(
			if getFileAttribute str_exportFileName #readonly == false then
			(
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
				append str_oldExports (str_fileName + "\n\t")
			)
			else
			(
				setFileAttribute str_exportFileName #readonly false
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 	
				append str_oldExports (str_fileName + "\n\t")
				append str_roFiles (str_fileName + "\n\t")
			)
		)
		else
		(
			exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
			append str_newExports (str_fileName + "\n\t")
		)
	)
	
	fn f_exportFile_TimeTag str_tagName str_exportFileName str_fileName str_oldExports str_newExports str_roFiles =
	(
		if doesFileExist str_exportFileName then
		(
			if getFileAttribute str_exportFileName #readonly == false then
			(
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
				format "File exported ok... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
				append str_oldExports (str_fileName + "\n\t")
			)
			else --// Make it writable
			(
				setFileAttribute str_exportFileName #readonly false
				exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 	
				format "File was readonly and overwritten... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
				append str_oldExports (str_fileName + "\n\t")
				append str_roFiles (str_fileName + "\n\t")
			)
		)
		else --// First time creation so just export it
		(
			exportFile str_exportFileName #noPrompt selectedOnly:true using:FBXEXP 
			format "File exported ok... % ... Range:%-%(%)\n" (str_tagName + ".FBX") ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
			append str_newExports (str_fileName + "\n\t")
		)
	)
	
	--// Reset the log file string contents
	g_LBTools_FBX.str_outputToLog = ""
	
	if execute (str_sameFolder) then
	(
		--// Output to the current maxfile scene path
		if maxFilePath != "" then
		(
			str_outputPath = substring maxFilePath 1 (maxFilePath.count - 1)
		)
		else
		(
			str_outputPath = getDir #scene
		)
	)
	else --// Read the ini file and extract the output path from that
	(
		str_outputPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
	)

	if (getDirectories str_outputPath).count > 0 then --// It's a valid output path, so continue
	(		
		if g_LBTools_FBX.f_useSameFolder() do
		(
			--// Listener output
			format "==========================================================\n// Skeletal Export Summary - %\n----------------------------------------------------------\n" localTime
			format "Scene file... %\n" (maxFilePath + maxFileName)
			
			str_outputPath = str_outputPath + @"\"
			
			local a_skinObjs = #()
			local a_selObjs = getCurrentSelection()
			
			--// Open all groups in case there are LoD's or other non-LoD groups. The individual meshes must be accessible (opened group) in order to find the export skeleton bones
			for obj in objects where (isGroupMember obj) do
			(
				setGroupOpen obj.parent true 
			)
			
			--// Finds all non-LoD skeletal meshes and LoD skeletal meshes (but only one per group set is needed)
			fn f_collectSkinMeshes a_skinObjs a_collection obj =
			(
				local a_groupLODs = #() --// Keep track of a single LOD mesh per group
				for obj in a_collection where (g_LBTools_FBX.f_findSkinModifier obj) do
				(
					if isGroupMember obj AND classOf (getVisController(obj)) == LOD_Controller then --// Is part of a LoD set
					(
						--// Have we already found a member of this group LoD set
						local found = false
						for o in a_groupLODs do
						(
							if obj.parent == o.parent do
							(
								found = true
								break
							)
						)
						if NOT found do --// A unique member of a LoD set, so add it to skin objects and track it
						(
							append a_groupLODs obj
							append a_skinObjs obj
						)
					)
					else --// Not a group LoD (i.e. a stand-alone skeletal mesh)
					(
						append a_skinObjs obj
					)
				)
			)
			
			--// Find all skin meshes depending on scene selection
			if a_selObjs.count > 0 then --// Find only valid skinned meshes in the selection
			(
				f_collectSkinMeshes a_skinObjs a_selObjs obj
				if a_skinObjs.count == 0 do --// No skin meshes found in the selection, so search the scene instead
				(
					f_collectSkinMeshes a_skinObjs geometry obj	
				)
			)
			else --// Search the scene for valid skin meshes
			(
				f_collectSkinMeshes a_skinObjs geometry obj
			)
			
			--// DEBUG
-- 			for s in a_skinObjs do format "% | %\n" s.name (if s.parent !=undefined then s.parent.name else undefined)
			
			--// Export valid skeletal meshes or animation
			if a_skinObjs.count > 0 then
			(
				if str_skeletalMode == "3" then --// GESTALT : Export separate meshes as a combined mesh per hierarchy (can be multiple hierarchies, Unreal will split them on import from a single FBX).
				(
					fbxExporterSetParam "Animation" false
					
					--// Search for selection sets with the prefix tag "FBX_" followed by the output name for that group of objects (e.g. FBX_Character_Body_Var_1)
					local flag_validSSfound = false
					local a_ssForExport = #()
					
					local ssCount = selectionSets.count
					if ssCount > 0 do
					(
						for i = 1 to ssCount do
						(
							if matchpattern selectionSets[i].name pattern:"FBX_*" do
							(
								append a_ssForExport selectionSets[i].name
								flag_validSSfound = true
							)
						)
					)
					
					if flag_validSSfound AND (selection.count == 0) then --// With nothing selected it uses the specially tagged selection sets to export from.
					(
						for i = 1 to a_ssForExport.count do
						(
							local str_outputName = subString a_ssForExport[i] 5 a_ssForExport[i].count --// Remove the FBX_ prefix from the output name
							
							select selectionSets[i]
							local a_selObjs = getCurrentSelection()
							local a_skinObjs = #()
							
							if a_selObjs.count > 0 do
							(
								--// Find only valid skinned meshes in the selection set
								for obj in a_selObjs do
								(
									if (g_LBTools_FBX.f_findSkinModifier obj) do
									(
										append a_skinObjs obj
									)
								)
							)
							
							--// Select all the skinned meshes and their full export rig
							clearSelection()
							select a_skinObjs[1]
							g_LBTools_FBX.f_selectSkinBones selection[1]
							g_LBTools_FBX.f_selectHierarchy selection[1]
							selectmore a_skinObjs

							--// Export the file
							str_fileName = (str_outputName + ".FBX")
							str_exportFileName = (str_outputPath + str_fileName)
							f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
							
							--// Listener output
							format "Gestalt Skeletal Mesh export...\n"
							format "File exported ok... %\n" ((getFileNameFile str_exportFileName) + ".FBX")
						)
						
						--// Display export summary
						local str_results = ("=== Export Summary ===\n\n" + "New: \n\t" + str_newExports + "\n" + "Overwritten: \n\t" + str_oldExports + "\n" + "Reconcile: \n\t" + str_roFiles)
						messagebox str_results title:"LB Tools"
					)
					else --// Use the previously found skin objects (either selected or just what's in the scene) to export from
					(
						local str_outputName = (getFilenameFile maxfilename)
						
						--// Select all the skinned meshes and the full export rig
						clearSelection()
						select a_skinObjs[1]
						g_LBTools_FBX.f_selectSkinBones selection[1]
						g_LBTools_FBX.f_selectHierarchy selection[1]
						selectmore a_skinObjs
						
						--// Export the file
						str_fileName = (str_outputName + ".FBX")
						str_exportFileName = (str_outputPath + str_fileName)
						f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
						
						--// Listener output
						format "Gestalt Skeletal Mesh export...\n"
						format "File exported ok... %\n" ((getFileNameFile str_exportFileName) + ".FBX")
						
						--// Display export summary
						local str_results = ("=== Export Summary ===\n\n" + "New: \n\t" + str_newExports + "\n" + "Overwritten: \n\t" + str_oldExports + "\n" + "Reconcile: \n\t" + str_roFiles)
						messagebox str_results title:"LB Tools"
					)
					
					fbxExporterSetParam "Animation" true
				)
				else --// Export as either separate skeletal meshes or animation
				(
					local okToExport = false
					if str_skeletalMode == "4" then --// ANIMATED : Perform a few checks to see what can be exported for animation
					(
						if a_skinObjs.count == 1 then --// Only one skin mesh in the scene or selection, so good to export
						(
							okToExport = true
						)							
						else --// Manage more than one skin mesh in the scene or selection
						(
							if selection.count == 1 then --// User has a single object selected, so check if it's one of the skin meshes already found
							(
								local foundValidSkinMesh = false
								for m in a_skinObjs do
								(
									if selection[1] == m do
									(
										foundValidSkinMesh = true
										okToExport = true
										exit
									)
								)
								if foundValidSkinMesh == false do
								(
									messageBox "Error: Selected object is not a valid skin mesh." title:""
								)
							)
							else --// Either nothing is selected or more than one object is selected
							(
								if str_skinNameGeneral != "*** BLANK ***" AND str_skinNameGeneral != "" then --// A skin mesh name has been provided, so check it's a valid skin mesh node
								(
									local meshNode = getNodeByName str_skinNameGeneral
									if meshNode != undefined then --// Valid node found
									(
										if (g_LBTools_FBX.f_findSkinModifier meshNode) then --// Check if it's a skin node
										(
											--// Replace the existing meshes in the array, so only this one gets used for exporting
											a_skinObjs = meshNode
											okToExport = true
										)
										else
										(
											messageBox "Error: The object for the skin mesh name provided exists in the scene, but is not a valid skin mesh." title:""
										)
									)
									else --// Name not a valid node
									(
										messageBox "Error: Skin mesh name provided is not a valid node in the current scene." title:""
									)									
								)
								else --// Skin mesh name was not entered and multiple skin meshes exist in the scene with nothing selected
								(
									--// Check for multiple hierarchies which are associated with the found skin meshes - if only one rig is found after the comparison search then use it
									g_LBTools_FBX.f_selectSkinBones a_skinObjs[1]
									g_LBTools_FBX.f_selectHierarchy selection[1]
									local a_firstBones = selection as array
									local multipleFound = false
									for i = 2 to a_skinObjs.count do
									(
										g_LBTools_FBX.f_selectSkinBones a_skinObjs[i]
										g_LBTools_FBX.f_selectHierarchy selection[1]
										local a_compareBones = selection as array
										for i = 1 to a_firstBones.count do
										(
											if a_firstBones[i] != a_compareBones[i] do --// Assume you've found a new rig
											(
												multipleFound = true
												exit
											)
										)
										if multipleFound do
										(
											exit
										)
									)
									clearSelection()
									if multipleFound == false then --// Treat the multiple skin meshes as part of the same rig for export purposes
									(
										a_skinObjs = a_skinObjs[1] --// We only need one of the shared rig meshes to pass down for exporting with
										okToExport = true
									)
									else --// Multiple hierarchies are in the scene
									(
										--// Display a dialog to select a skin mesh from a list or names and re-run the skeletal process
										f_selectSkinMeshUI a_skinObjs
										--// Flag to stop the scene fetching in order to keep objects unhidden and unfrozen for selection
										g_LBTools_FBX.skinMeshSelectProcess = true
										return undefined --// Abort the current process at this point
									)
								)
							)
						)
					)
					else --// ok to export separate skeletal meshes as Animated was off
					(
						okToExport = true
					)
					
					if okToExport do
					(
						sliderTime += 1 --// This is to update any animation CA's or controllers
						for obj in a_skinObjs do
						(
							local str_outputName
							if str_skeletalMode == "4" then --// ANIMATED
							(
								str_outputName = (getFilenameFile maxfilename)
							)
							else
							(
								if isGroupMember obj AND classOf (getVisController(obj)) == LOD_Controller then
								(
									str_outputName = obj.parent.name --// Name of the group header dummy
								)
								else
								(
									str_outputName = obj.name
								)
							)
							
							--// Select all the skinned meshes and their export rig, either partial or full
							clearSelection()
							g_LBTools_FBX.f_selectSkinBones obj
							if str_skeletalMode != "2" do --// PARTIAL RIG : The partial rig mode uses the skin bones only, but FBX will automatically include all parent bones up to the root
							(
								g_LBTools_FBX.f_selectHierarchy selection[1]
							)
							if isGroupMember obj AND classOf (getVisController(obj)) == LOD_Controller then
							(
								local groupHead = (getNodeByName obj.parent.name)
								setGroupOpen groupHead false --// Close the group
								selectmore groupHead --// Select a LoD and its set members, as well as the skin bones
							)
							else
							(
								selectmore obj
							)
							
							--// Deselect any skin meshes if exporting animation
							if str_skeletalMode == "4" do --// ANIMATED
							(
								for obj in (selection as array) do
								(
									if (g_LBTools_FBX.f_findSkinModifier obj) do
									(
										deselect obj
									)
								)
							)
							
							--// Check if there are any time tags in the scene for composite animations
							local ttCount = FrameTagManager.GetTagCount()
							if str_skeletalMode == "4" AND ttCount > 0 AND (execute (str_ignoreTimeTags) == false) then --// ANIMATED
							(
								if exportFBX_Rollout.chkbx_useTT.checked then --// Export only a single sequence using the selected item in the time tags dropdown list
								(
									if exportFBX_Rollout.ddl_timeTags.items.count == 0 then
									(
										exportFBX_Rollout.f_populateTimeTagsList()
										messageBox "Error: Time tags were found, but the dropdown list was empty.\n\nTry exporting again." title:""
									)
									else
									(
										try --// Adding a Try Catch to prevent crashes when time tags are not valid
										(
											--// Listener output
											format "Animation (time tags) export...\n"
											
											local aaRange = animationRange

											--//******************************************************************************************************************************************************
											--// NOTE:
											--// The FBX exporter will respect all keys on what you have selected during the export. This means you can't have keys existing outside of the time range as it will
											--// inlcude those keys as well. Instead you need to have your export rig indirectly controlled using constraints with no keys then time ranges will be respected. This
											--// is neccessary for the time tags method of exporting.
											--//******************************************************************************************************************************************************
											
											--// Read the time tag to extract the name and range specified
											local str_tt = FrameTagManager.GetNameByID exportFBX_Rollout.ddl_timeTags.selection
											local a_tt = (filterString str_tt " ")
											local str_tagName = a_tt[1]
											local a_range = (filterString a_tt[2] "-")
											animationRange = interval (a_range[1] as time) (a_range[2] as time)
											
											g_LBTools_FBX.f_initializeFBXexport()
											
											--// Export the file and handle read only
											str_fileName = (str_tagName + ".FBX")
											str_exportFileName = (str_outputPath + str_fileName)
											f_exportFile_TimeTag str_tagName str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
											
											animationRange = aaRange
											sliderTime = 0
										)
										catch
										(
											messageBox "Error: There's a problem exporting using the scene Time Tags.\n\nCheck the formatting of them." title:""
											format "Error: Scene >> % << may have invalid time tags.\n" maxFileName
										)
									)
								)
								else --// Export all time tag sequences
								(
									try --// Adding a Try Catch to prevent crashes when time tags are not valid
									(
										--// Listener output
										format "Animation (time tags) export...\n"
										
										local aaRange = animationRange
										for i = 1 to ttCount do
										(
											--//******************************************************************************************************************************************************
											--// NOTE:
											--// The FBX exporter will respect all keys on what you have selected during the export. This means you can't have keys existing outside of the time range as it will
											--// inlcude those keys as well. Instead you need to have your export rig indirectly controlled using constraints with no keys then time ranges will be respected. This
											--// is neccessary for the time tags method of exporting.
											--//******************************************************************************************************************************************************
											
											--// Read the time tag to extract the name and range specified
											local str_tt = FrameTagManager.GetNameByID i
											local a_tt = (filterString str_tt " ")
											local str_tagName = a_tt[1]
											local a_range = (filterString a_tt[2] "-")
											animationRange = interval (a_range[1] as time) (a_range[2] as time)
											
											g_LBTools_FBX.f_initializeFBXexport()
											
											--// Export the file and handle read only
											str_fileName = (str_tagName + ".FBX")
											str_exportFileName = (str_outputPath + str_fileName)
											f_exportFile_TimeTag str_tagName str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
										)
										animationRange = aaRange
										sliderTime = 0
									)
									catch
									(
										messageBox "Error: There's a problem exporting using the scene Time Tags.\n\nCheck the formatting of them." title:""
										format "Error: Scene >> % << may have invalid time tags.\n" maxFileName
									)
								)
							)
							else --// No time tags found, or not an animation file
							(
								--// Listener output
								if str_skeletalMode == "4" then --// ANIMATED
								(
									format "Animation export...\n"
								)
								else --// FULL RIG or PARTIAL RIG
								(
									fbxExporterSetParam "Animation" false
									format "Skeletal Mesh export...\n" 
								)
								
								--// Check if the user has entered an animation name for output
								if str_skeletalMode == "4"AND str_animName != "*** BLANK ***" AND str_animName != "" do
								(
									str_outputName = str_animName
								)
								
								--// Export the file
								str_fileName = (str_outputName + ".FBX")
								str_exportFileName = (str_outputPath + str_fileName)
								f_exportFile str_exportFileName str_fileName str_oldExports str_newExports str_roFiles
								
								--// Listener output
								if str_skeletalMode == "4" then --// ANIMATED
								(	
									format "File exported ok... % ... Range:%-%(%)\n" str_fileName ((animationRange.start) as integer / ticksperframe) ((animationRange.end) as integer / ticksperframe) ((animationRange.end - animationRange.start + 1) as integer / ticksperframe)
								)
								else
								(
									format "File exported ok... %\n" ((getFileNameFile str_exportFileName) + ".FBX")
								)
							)
						)
						
						fbxExporterSetParam "Animation" true
						
						--// Display export summary
						local str_results = ("=== Export Summary ===\n\n" + "New: \n\t" + str_newExports + "\n" + "Overwritten: \n\t" + str_oldExports + "\n" + "Reconcile: \n\t" + str_roFiles)
						messagebox str_results title:"LB Tools"
					)
				)
			)
			else
			(
				messageBox "Error: No valid skin meshes were found in the scene or selection." title:""
			)
			local str_sceneFile = (maxfilepath + maxfilename)
			format "Export Path... %\n----------------------------------------------------------\n" str_outputPath
			format "// Skeletal Export Completed - %\n==========================================================\n" localTime
		)
	)
	else
	(
		messageBox "Error: Export Path not valid." title:""
	)
), --// end function

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_exportFBX_UI =
(
	rollout exportFBX_Rollout "LB Tools"
	(
		fn f_returnItemPaths txtFile =
		(
			--// Convert the full paths stored in the text file which include the Root Path substring at the beginning, into shorter path substrings for dropdown list display.
			--// The function expects the text file to exist and be passed in.
			
			g_LBTools_FBX.str_rootPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
			if g_LBTools_FBX.str_rootPath.count > 0 AND g_LBTools_FBX.str_rootPath != "*** BLANK ***" then
			(
				local a_paths = #()
				local textFileRead = openFile txtFile mode:"r"
				while NOT eof textFileRead do
				(
					try(append a_paths (replace (readLine textFileRead) 1 g_LBTools_FBX.str_rootPath.count ""))catch()
				)
				close textFileRead
				if a_paths.count == 0 do
				(
					append a_paths @"\*** The text file is empty OR the root path is invalid for stored paths ***"
				)
				a_paths
			)
			else
			(
				#(@"\*** INVALID ROOT PATH - USE SETTINGS TO CHANGE ***")
			)
		)
		
		fn f_populateTimeTagsList =
		(				
			local ttCount = FrameTagManager.GetTagCount()
			if ttCount > 0 then
			(
				local a_tt_Items = #()
				for i = 1 to ttCount do
				(
					local str_tt = FrameTagManager.GetNameByID i
					local a_tt = (filterString str_tt " ")
					local str_tagName = a_tt[1]
					append a_tt_Items str_tagName
				)
				exportFBX_Rollout.ddl_timeTags.items = a_tt_Items
				exportFBX_Rollout.ddl_timeTags.height = (a_tt_Items.count + 1) * 100
			)
			else
			(
				exportFBX_Rollout.ddl_timeTags.items = #()
			)
		)
		
		fn f_removeEmptySelectionSets ssCount =
		(
			local a_deleteItems = #()
			for setIndex = 1 to ssCount do
			(
				if (getNamedSelSetItemCount setIndex) == 0 do
				(
					append a_deleteItems (getNamedSelSetName setIndex)
				)
			)
			for str_setName in a_deleteItems do
			(
				deleteItem selectionSets str_setName
				exportFBX_Rollout.ddl_SS.selection = 1
			)
		)
		
		fn f_populateList_SS =
		(
			local a_ss_Items = #()
			local ssCount = getNumNamedSelSets()
			if ssCount > 0 then
			(
				f_removeEmptySelectionSets ssCount
				ssCount = getNumNamedSelSets()
				if ssCount > 0 then
				(
					exportFBX_Rollout.chkbx_useStaticSS.enabled = true
					for i = 1 to ssCount do
					(
						append a_ss_Items (getNamedSelSetName i)
					)
					exportFBX_Rollout.ddl_SS.items = a_ss_Items
					exportFBX_Rollout.ddl_SS.height = (a_ss_Items.count + 1) * 100
				)
				else
				(
					exportFBX_Rollout.ddl_SS.items = #()
					exportFBX_Rollout.chkbx_useStaticSS.checked = false
					exportFBX_Rollout.chkbx_useStaticSS.enabled = false
				)
			)
			else
			(
				exportFBX_Rollout.ddl_SS.items = #()
				exportFBX_Rollout.chkbx_useStaticSS.checked = false
				exportFBX_Rollout.chkbx_useStaticSS.enabled = false
			)
		)
		
		-------------------------------------------------------------------------------------------
		--// Rollout definitions and event handlers
		-------------------------------------------------------------------------------------------
		
		local imageFile1 = (getDir #usericons) + "\LBTools_ExportFBX_Image1.bmp" --// Static and Skeletal
		local imageFile2 = (getDir #usericons) + "\LBTools_ExportFBX_Image2.bmp" --// Batch Animation {icon is an illustration from Physics for Entertainment by Yakov Isidorovich Perelman from archive.org}
		local imageFile3 = (getDir #usericons) + "\LBTools_Guide_Image.bmp"
		local imageFile4 = (getDir #usericons) + "\LBTools_Folder_Image.bmp"
		local imageFile5 = (getDir #usericons) + "\LBTools_Gear_Image.bmp"
		
		button btn_FBXexportSettings "Settings" align:#left across:5 offset:[0,0] images:#(imageFile5, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens a new dialog which allows for setting up of the Root Path used by the text file dropdown list, as well as the FBX export geometry settings."
		button btn_deleteIniFiles "x" width:16 height:16 align:#left offset:[-50,0] tooltip:"Resets the interface settings by deleting the INI files."
		button btn_titleDecal "" width:245 height:30 align:#center offset:[10,5] enabled:false border:false
		label lbl_guide "" align:#right offset:[53,10]
		button btn_guide "" align:#right offset:[2,0] images:#(imageFile3, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an online guide explaining the tool's features."
		
		group "Path Settings"
		(
			edittext edtxt_exportPath "Export Path:" width:378 across:3 align:#left
			button btn_clearText1 "x" align:#right offset:[116,2] width:16 height:16 tooltip:"Clears the text from the Export Path edit box."
			button btn_browsePath "[...]" width:26 offset:[5,0] align:#right tooltip:"Browse for an Export Path."
			
			label lbl_path "Choose a path stored in a text file to replace the current Export Path..." align:#left offset:[0,5] across:3
			button btn_folderExportPath "" align:#right offset:[112,0] images:#(imageFile4, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an Explorer window showing the Export Path."
			button btn_reloadTextFileItems "R" align:#right offset:[0,3] width:20 tooltip:"Repopulate the dropdown list from the text file after making changes to it."
			
			dropdownList ddl_textFile "" width:395 align:#left across:2 tooltip:"Paths stored in a text file and chosen from the dropdown list can be loaded into the Export Path."
			button btn_openTextFile "TXT" align:#right offset:[5,0] width:26 tooltip:"Opens the text file C:\\temp\\LBTools_ExportPaths.txt for editing stored export paths that appear in the dropdown list."
			
			checkbox chkbx_sameFolder "Same folder as scene file" tooltip:"This option overrides the Export Path and outputs the FBX files to the same folder as the currently open scene file."
		)
		
		label lbl_gap1 ""
		button btn_exportFBX_Static "Static" width:195 height:50 offset:[-5,-10] across:2 images:#(imageFile1, undefined, 2, 1, 1, 1, 1, false) tooltip:"Export static meshes and their associated collision volumes (if any)."
		button btn_exportFBX_Skeletal "Skeletal" width:195 height:50 offset:[5,-10] images:#(imageFile1, undefined, 2, 2, 2, 1, 1, false) tooltip:"Export skeletal meshes or animations. Looks for the Skin modifier and full hierarchy."
		
		checkbox chkbx_singleMesh "Single Mesh (attached meshes)" align:#left across:2 offset:[0,0] tooltip:"Attaches the selection into a single mesh: dupliactes the selection, collapses to a single object, exports to a single FBX file, deletes the duplicate mesh. Does not report to the log file and has no collisions."
		radiobuttons rdobtns_skeletalMode "" labels:#("Full Rig","Partial Rig","Gestalt","Animation") columns:4 align:#right offsets:#([0,0],[0,0],[10,0],[-175,20]) offset:[80,0] tooltip:#("Exports the skeletal meshes using the entire hierarchy as separate files.","Exports the skeletal meshes using only the assigned Skin bones (FBX includes all parent bones by default) as separate files.","Exports the skeletal meshes as a single file or uses FBX_ tagged selection sets.","Exports only the animation on the export rig - excludes meshes.")
		
		checkbox chkbx_centreAtOrigin "Centre At Origin" align:#left offset:[0,-20] tooltip:"Each object will be relocated to the origin.\n\nNOTE: Static mesh collision volumes must be linked to follow the object."
			
		checkbox chkbx_orientWorld "Orient World" across:3 align:#left offset:[25,0] enabled:true tooltip:"Each object will reorient to the world - rotation [0,0,0]."
		checkbox chkbx_centrePivot "Centre Pivot" align:#left offset:[-25,0] enabled:true tooltip:"Each object will have it's pivot moved to the centre of the geometry."
		label lbl_skinNameGeneral "Skin mesh name for animation export:" align:#right offset:[-16,0]
		
		checkbox chkbx_singleFBX "Output to a single FBX file" align:#left offset:[0,0] across:3 tooltip:"All static meshes and collisions found will be saved into a single FBX file. The output name will be the maxfile name with the added prefix 'StaticMeshes_'."
		edittext edtxt_skinNameGeneral "" align:#right width:188 offset:[126,-3] across:2 tooltip:"Use a specified name as the skinned mesh to export from, in case there's multiple skinned meshes in the scene. If left blank, it uses the first skinned mesh found."
		button btn_clearText2 "x" align:#right offset:[3,-3] width:16 height:16 tooltip:"Clears the text from the Skin Mesh Name edit box."
		
		checkbox chkbx_useStaticSS "Selection Set:" checked:false align:#left offset:[0,5] across:4 tooltip:"Use a specific selection set to get static meshes to export from. The selection set does not have to contain the collision volumes."
		button btn_ssRefresh "r" align:#right offset:[-8,5] width:16 height:16 tooltip:"Refresh the interface selection sets dropdown list taken from the scene."
		label lbl_timeTags "Time Tags:" align:#right offset:[-41,5]
		checkbox chkbx_ignoreTimeTags "Ignore time tags" align:#right offset:[-35,4] tooltip:"Used with Animation and Batch Animation only: Exports a single animation FBX file using the maxfile name, irrespective of any existing time tags."
		
		dropdownList ddl_SS "" width:185 align:#left offset:[0,-3] across:6 tooltip:"Pick a selection set to get the static meshes for exporting."
		button btn_deleteSS "x" width:16 height:16 align:#left offset:[118,0] tooltip:"Deletes all selection sets in the scene."
		checkbox chkbx_useTT "" checked:false align:#right offset:[35,0] tooltip:"Export only the sequence from the selected time tag in the dropdown list."
		dropdownList ddl_timeTags "" width:145 align:#right offset:[110,-3] tooltip:"Select a time tag to export only that sequence."
		button btn_deleteTimeTags "x" width:16 height:16 align:#right offset:[59,0] tooltip:"Deletes all time tags in the scene."
		button btn_ttRefresh "r" align:#right offset:[7,0] width:16 height:16 tooltip:"Refresh the interface time tags dropdown list taken from the scene."
		
		label lbl_animName "Output name for animation export:" align:#right offset:[-30,0]
		
		checkbox chkbx_displayLogFile "Display Log File" align:#left offset:[0,-5] across:4 tooltip:"Used to display a text file summary of static mesh exporting and their associated collisions."
		button btn_openLogFile "Open Log File" offset:[0,-8] width:80 height:22 align:#left tooltip:"Opens the log file in a text editor."
		edittext edtxt_animName "" align:#right width:188 offset:[90,0] tooltip:"Use a specified name as the FBX output name. If left blank, it uses the maxfile name instead. Time tags will override this function." across:2
		button btn_clearText3 "x" align:#right offset:[2,0] width:16 height:16 tooltip:"Clears the text from the Animation Name edit box."
		
		groupBox gpbx_1 "" pos:[4,162] width:218 height:237
		groupBox gpbx_2 "" pos:[227,162] width:218 height:237
		
		group "Batch Animation Exporting"
		(
			checkbox chkbx_selectedMaxfiles "Selected maxfiles" align:#left across:2 tooltip:"Displays a dialog showing a name listing of all maxfiles in the chosen path which can then be selected for batch exporting."
			checkbox chkbx_recurseSubfolders "Recurse subfolders" align:#left offset:[-100,0] tooltip:"Exports all maxfiles found in a picked root path including subfolders. Uses the Export Path as an output root path and creates folders based on the maxfile's subfolder structure."
			label lbl_skinNameBatch "Skin mesh name for batch animation export:" align:#left offset:[2,0]
			edittext edtxt_skinNameBatch "" align:#left width:200 offset:[-5,0] across:3 tooltip:"Use a specified name as the skinned mesh to export from, in case there's multiple skinned meshes in the scene. If left blank, it uses the first skinned mesh found."
			button btn_clearText4 "x" align:#left offset:[58,0] width:16 height:16 tooltip:"Clears the text from the Skin Mesh Name edit box."
			button btn_batchAnims "Batch Animations" width:174 height:38 align:#left offset:[-45,-30] images:#(imageFile2, undefined, 1, 1, 1, 1, 1, false) tooltip:"Export skeletal animations from a collection of maxfiles based off a chosen maxfiles folder to the specified Export Path."
		)
		group "Shift + Left Click ---> Non-UI Execution (uses existing settings)"
		(
			radioButtons rdobtns_shiftClick "" labels:#("Static","Skeletal")	align:#center offsets:#([-20,0],[20,0]) tooltip:#("Quickly export using the STATIC method when Shift Left Clicking on the Export FBX toolbar or menu items.","Quickly export using the SKELETAL method when Shift Left Clicking on the Export FBX toolbar or menu items.")
		)
		
		button btn_about "About" align:#center
		
		on btn_about pressed do
		(
			try(destroyDialog g_LBTools_FBX.about_Rollout)catch()
			createDialog g_LBTools_FBX.about_Rollout
		)
		
		on exportFBX_Rollout moved val do
		(
			g_LBTools_FBX.dPos = val
		)
		
		on exportFBX_Rollout close do
		(
			if g_LBTools_FBX.FBXsettings_rollout != undefined do
			(
				if g_LBTools_FBX.FBXsettings_rollout.open do
				(
					try(destroyDialog g_LBTools_FBX.FBXsettings_rollout)catch()
				)
			)
			if g_LBTools_FBX.maxFileSelection_rollout != undefined do
			(
				if g_LBTools_FBX.maxFileSelection_rollout.open do
				(
					try(destroyDialog g_LBTools_FBX.maxFileSelection_rollout)catch()
				)
			)
			if g_LBTools_FBX.selectSkinMesh_Rollout != undefined do
			(
				if g_LBTools_FBX.selectSkinMesh_Rollout.open do
				(
					try(destroyDialog g_LBTools_FBX.selectSkinMesh_Rollout)catch()
					fetchMaxFile quiet:true
				)
			)
			try(destroyDialog g_LBTools_FBX.about_Rollout)catch()
		)
		
		on btn_deleteIniFiles pressed do
		(
			if (queryBox "WARNING: Are you sure you want to delete all INI files and reset the interface?" title:"") do
			(
				deleteFile g_LBTools_FBX.exporterINI
				deleteFile g_LBTools_FBX.FBXexportINI
				g_LBTools_FBX.f_createFBXexportINI()
				g_LBTools_FBX.f_exportFBX_UI()
			)
		)
		
		on exportFBX_Rollout open do
		(			
			if NOT (doesFileExist g_LBTools_FBX.exporterINI) then --// Setup initial UI settings for the first time
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath" @"*** BLANK ***"
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" @"C:\Program Files"
				edtxt_exportPath.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral" "*** BLANK ***"
				edtxt_skinNameGeneral.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral"
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName" "*** BLANK ***"
				edtxt_animName.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName"
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch" "*** BLANK ***"
				edtxt_skinNameBatch.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch"
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog" "true"
				chkbx_displayLogFile.enabled = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder" "false"
				chkbx_sameFolder.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh" "false"
				chkbx_singleMesh.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX" "false"
				chkbx_singleFBX.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags" "false"
				chkbx_ignoreTimeTags.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin" "true"
				chkbx_centreAtOrigin.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld" "true"
				chkbx_orientWorld.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot" "false"
				chkbx_centrePivot.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode" "1"
				rdobtns_skeletalMode.state = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles" "false"
				chkbx_selectedMaxfiles.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders" "false"
				chkbx_recurseSubfolders.checked = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders")				
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick" "1"
				rdobtns_shiftClick.state = execute (getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick")
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme" "1"
				
				g_LBTools_FBX.f_openExporterUI()
			)
			else --// Update the UI using the stored INI settings
			(
				local flag_updateUI = false
				
				local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
				if str_exportPath == "" do
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath" @"*** BLANK ***"
					flag_updateUI = true
				)
				
				local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
				if str_exportPath != "" then
				(
					edtxt_exportPath.text = str_exportPath
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" @"C:\Program Files"
					edtxt_exportPath.text = @"C:\Program Files"
					flag_updateUI = true
				)

				local str_skinNameGeneral = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral"
				if str_skinNameGeneral != "" then
				(
					edtxt_skinNameGeneral.text = str_skinNameGeneral
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral" "*** BLANK ***"
					edtxt_skinNameGeneral.text = "*** BLANK ***"
					flag_updateUI = true
				)
				
				local str_animName = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName"
				if str_animName != "" then
				(
					edtxt_animName.text = str_animName
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName" "*** BLANK ***"
					edtxt_animName.text = "*** BLANK ***"
					flag_updateUI = true
				)
				
				local str_skinNameBatch = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch"
				if str_skinNameBatch != "" then
				(
					edtxt_skinNameBatch.text = str_skinNameBatch
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch" "*** BLANK ***"
					edtxt_skinNameBatch.text = "*** BLANK ***"
					flag_updateUI = true
				)
				
				local str_displayLog = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog"
				if str_displayLog != "" then
				(
					chkbx_displayLogFile.checked = execute (str_displayLog)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog" "true"
					chkbx_displayLogFile.checked = true
					flag_updateUI = true
				)
				
				local str_sameFolder = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder"
				if str_sameFolder != "" then
				(
					chkbx_sameFolder.checked = execute (str_sameFolder)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder" "false"
					chkbx_sameFolder.checked = false
					flag_updateUI = true
				)
				
				local str_singleMesh = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh"
				if str_singleMesh != "" then
				(
					chkbx_singleMesh.checked = execute (str_singleMesh)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh" "false"
					chkbx_singleMesh.checked = false
					flag_updateUI = true
				)

				local str_singleFBX = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX"
				if str_singleFBX != "" then
				(
					chkbx_singleFBX.checked = execute (str_singleFBX)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX" "false"
					chkbx_singleFBX.checked = false
					flag_updateUI = true
				)
				
				local str_ignoreTimeTags = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags"
				if str_ignoreTimeTags != "" then
				(
					chkbx_ignoreTimeTags.checked = execute (str_ignoreTimeTags)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags" "false"
					chkbx_ignoreTimeTags.checked = false
					flag_updateUI = true
				)
				
				local str_centreAtOrigin = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin"
				if str_centreAtOrigin != "" then
				(
					chkbx_centreAtOrigin.checked = execute (str_centreAtOrigin)
					if chkbx_centreAtOrigin.checked == false do
					(
						chkbx_orientWorld.enabled = false
						chkbx_centrePivot.enabled = false
					)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin" "true"
					chkbx_centreAtOrigin.checked = false
					flag_updateUI = true
				)
				
				local str_orientWorld = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld"
				if str_orientWorld != "" then
				(
					chkbx_orientWorld.checked = execute (str_orientWorld)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld" "true"
					chkbx_orientWorld.checked = false
					flag_updateUI = true
				)
				
				local str_centrePivot = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot"
				if str_centrePivot != "" then
				(
					chkbx_centrePivot.checked = execute (str_centrePivot)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot" "false"
					chkbx_centrePivot.checked = false
					flag_updateUI = true
				)
				
				local str_skeletalMode = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode"
				if str_skeletalMode != "" then
				(
					rdobtns_skeletalMode.state = execute (str_skeletalMode)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode" "1"
					rdobtns_skeletalMode.state = 1
					flag_updateUI = true
				)				
				
				local str_selectedMaxfiles = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles"
				if str_selectedMaxfiles != "" then
				(
					chkbx_selectedMaxfiles.checked = execute (str_selectedMaxfiles)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles" "false"
					chkbx_selectedMaxfiles.checked = false
					flag_updateUI = true
				)

				local str_recurseSubfolders = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders"
				if str_recurseSubfolders != "" then
				(
					chkbx_recurseSubfolders.checked = execute (str_recurseSubfolders)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders" "false"
					chkbx_recurseSubfolders.checked = false
					flag_updateUI = true
				)
				
				local str_shiftClick = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick"
				if str_shiftClick != "" then
				(
					rdobtns_shiftClick.state = execute (str_shiftClick)
				)
				else
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick" "1"
					rdobtns_shiftClick.state = 1
					flag_updateUI = true
				)
				
				local str_titleTheme = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme"
				if str_titleTheme == "" do
				(
					setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme" "1"
					flag_updateUI = true
				)				
				
				--// Populate the items dropdown list for the export paths stored in a text file
				local txtFile = @"C:\Temp\LBTools_ExportPaths.txt"
				
				if NOT (doesFileExist txtFile) then --// If the file doesn't exist create it
				(
					g_LBTools_FBX.str_rootPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
					
					--// Create a new Export Paths text file
					makeDir @"C:\Temp"
					local edit_txtFile = openFile txtFile mode:"w"
					close edit_txtFile
					
					ddl_textFile.items = f_returnItemPaths txtFile
					ddl_textFile.height = (ddl_textFile.items.count + 1) * 100
				)
				else
				(
					ddl_textFile.items = f_returnItemPaths txtFile
					ddl_textFile.height = (ddl_textFile.items.count + 1) * 100
				)

				--// Setup the correct title decal based on the chosen color theme
				if getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "TitleTheme" == "1" then
				(
					g_LBTools_FBX.imageFileTitle = (getDir #usericons) + "\LBTools_ExportFBX_Image4.bmp" --// Title decal dark theme (default)
					btn_titleDecal.images = #(g_LBTools_FBX.imageFileTitle, undefined, 1, 1, 1, 1, 1, false)
				)
				else
				(
					g_LBTools_FBX.imageFileTitle = (getDir #usericons) + "\LBTools_ExportFBX_Image3.bmp" --// Title decal light theme
					btn_titleDecal.images = #(g_LBTools_FBX.imageFileTitle, undefined, 1, 1, 1, 1, 1, false)
				)
				
				--// Update the interface
				if flag_updateUI do
				(
					g_LBTools_FBX.f_openExporterUI()
				)

				--// Populate the static mesh selection sets list if any are found in the scene
				f_populateList_SS()
				
				--// Populate the time tags dropdown list if any are found in the scene
				f_populateTimeTagsList()
				
				setFocus edtxt_exportPath
			)
		)
		
		on btn_FBXexportSettings pressed do
		(
			g_LBTools_FBX.f_FBXexportSettingsUI()
		)
		
		on btn_clearText1 pressed do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" ""
			edtxt_exportPath.text = ""
			setFocus edtxt_exportPath
		)
		
		on btn_clearText2 pressed do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral" ""
			edtxt_skinNameGeneral.text = ""
			setFocus edtxt_skinNameGeneral
		)
		
		on btn_clearText3 pressed do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName" ""
			edtxt_animName.text = ""
			setFocus edtxt_animName
		)
		
		on btn_clearText4 pressed do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch" ""
			edtxt_skinNameBatch.text = ""
			setFocus edtxt_skinNameBatch
		)
		
		on ddl_textFile selected index do
		(
			--// NOTE: The Root Path must be a valid directoy. e.g. C:\Program Files {no backslash}
			g_LBTools_FBX.str_rootPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RootPath"
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" (g_LBTools_FBX.str_rootPath + ddl_textFile.items[index])
			edtxt_exportPath.text = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
			setFocus edtxt_exportPath
		)
		
		on btn_folderExportPath pressed do
		(
			local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
			if str_exportPath != "" AND (getDirectories str_exportPath).count > 0 then
			(
				ShellLaunch "explorer.exe" str_exportPath
			)
			else
			(
				messageBox "Error: Export Path specified is invalid." title:""
			)
		)
		
		on btn_reloadTextFileItems pressed do
		(
			local txtFile = @"C:\Temp\LBTools_ExportPaths.txt"
			if doesFileExist txtFile do
			(
				ddl_textFile.items = f_returnItemPaths txtFile
				ddl_textFile.height = (ddl_textFile.items.count + 1) * 100
			)
		)
		
		on btn_openTextFile pressed do
		(
			local txtFile = @"C:\Temp\LBTools_ExportPaths.txt"
			if doesFileExist txtFile do
			(
				ShellLaunch txtFile ""
			)
		)
		
		on btn_guide pressed do
		(
			shellLaunch "https://sites.google.com/view/lbtools-guide/home/export-fbx-to-unreal" ""
		)
		
		on btn_openLogFile pressed do
		(
			local txtFile = @"C:\Temp\LBTools_ExportLog.txt"
			if doesFileExist txtFile then
			(
				ShellLaunch txtFile ""
			)
			else
			(
				messageBox "Error: Log file does not exist yet, export a static mesh first with Display Log File checked." title:""
			)
		)
		
		on edtxt_exportPath changed str do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" str
		)

		on edtxt_skinNameGeneral changed str do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameGeneral" str
		)

		on edtxt_animName changed str do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "AnimName" str
		)
		
		on edtxt_skinNameBatch changed str do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkinNameBatch" str
		)
		
		on chkbx_useStaticSS changed theState do
		(
			f_populateList_SS()
		)
		
		on ddl_SS selected setIndex do
		(
			f_populateList_SS()
		)
		
		on btn_deleteSS pressed do
		(
			local ssCount = getNumNamedSelSets()
			if ssCount > 0 do
			(
				if (queryBox "Continue to delete all selection sets from the scene?" title:"") do
				(
					while ssCount > 0 do
					(
						deleteItem selectionSets ssCount
						ssCount -= 1
					)
					f_populateList_SS()
				)
			)
		)
		
		on btn_ssRefresh pressed do
		(
			f_populateList_SS()
		)
		
		on btn_deleteTimeTags pressed do
		(
			if FrameTagManager.GetTagCount() > 0 do
			(
				if (queryBox "Continue to delete all scene time tags?" title:"") do
				(
					FrameTagManager.ResetFrametags()
					g_LBTools_FBX.f_openExporterUI()
				)
			)
		)
		
		on btn_ttRefresh pressed do
		(
			f_populateTimeTagsList()
		)
		
		on btn_browsePath pressed do
		(
			local str_exportPath = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath"
			if str_exportPath == "" do
			(
				str_exportPath = maxFilePath
			)
			local str_pathToUse = getSavePath caption:"Choose a path to export to..." initialDir:str_exportPath
			if str_pathToUse != undefined do
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ExportPath" str_pathToUse
				edtxt_exportPath.text = str_pathToUse
				setFocus edtxt_exportPath
			)
		)

		on chkbx_sameFolder changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SameFolder" (theState as string)
		)
		
		on chkbx_singleMesh changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleMesh" (theState as string)
		)

		on chkbx_singleFBX changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SingleFBX" (theState as string)
		)
		
		on chkbx_ignoreTimeTags changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "IgnoreTimeTags" (theState as string)
		)
		
		on chkbx_displayLogFile changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "DisplayLog" (theState as string)
		)
		
		on chkbx_centreAtOrigin changed theState do
		(
			if theState then
			(
				exportFBX_Rollout.chkbx_orientWorld.enabled = true
				exportFBX_Rollout.chkbx_centrePivot.enabled = true
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin" "true"
			)
			else
			(
				exportFBX_Rollout.chkbx_orientWorld.enabled = false
				exportFBX_Rollout.chkbx_centrePivot.enabled = false
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentreAtOrigin" "false"
			)
		)

		on chkbx_orientWorld changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "OrientWorld" (theState as string)
		)		
		
		on chkbx_centrePivot changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "CentrePivot" (theState as string)
		)
		
		on rdobtns_skeletalMode changed theState do
		(		
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SkeletalMode" (theState as string)
		)
		
		on chkbx_selectedMaxfiles changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "SelectedMaxfiles" (theState as string)
			if g_LBTools_FBX.maxFileSelection_rollout != undefined do
			(
				if g_LBTools_FBX.maxFileSelection_rollout.open do
				(
					try(destroyDialog g_LBTools_FBX.maxFileSelection_rollout)catch()
				)
			)
		)

		on chkbx_recurseSubfolders changed theState do
		(
			if theState then
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders" (theState as string)
			)
			else
			(
				setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "RecurseSubfolders" (theState as string)
			)
			--// Update the maxfiles selection dialog if open
			if g_LBTools_FBX.maxFileSelection_rollout != undefined do
			(
				if g_LBTools_FBX.maxFileSelection_rollout.open do
				(
					g_LBTools_FBX.f_openSelectMaxfilesUI()
				)
			)
		)
		
		on rdobtns_shiftClick changed theState do
		(
			setIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick" (theState as string)
		)
	
		on btn_exportFBX_Static pressed do
		(
			if maxFilePath != "" then
			(
				g_LBTools_FBX.f_initializeFBXexport()
				holdMaxFile()
				unhide objects doLayer:true
				unfreeze objects doLayer:true
				g_LBTools_FBX.f_exportFBX_Static()
				fetchMaxFile quiet:true
			)
			else
			(
				messagebox "Error: Scene not saved to a file. Open or save a file to export from." title:""
			)			
		)
		
		on btn_exportFBX_Skeletal pressed do
		(
			if maxFilePath != "" then
			(
				g_LBTools_FBX.f_initializeFBXexport()
				holdMaxFile()
				unhide objects doLayer:true
				unfreeze objects doLayer:true
				g_LBTools_FBX.f_exportFBX_Skeletal()
				if g_LBTools_FBX.skinMeshSelectProcess == false do
				(
					fetchMaxFile quiet:true
				)
			)
			else
			(
				messagebox "Error: Scene not saved to a file. Open or save a file to export from." title:""
			)
		)	

		on btn_batchAnims pressed do
		(
			g_LBTools_FBX.f_initializeFBXexport()
			if g_LBTools_FBX.f_useSameFolder() do
			(
				--// Get the export path and check if it's valid	
				if g_LBTools_FBX.f_getBatchAnimExportPath() == false then
				(
					messageBox "Error: Export Path not valid." title:""
				)
				else
				(					
					--// Pick a folder of maxfiles to process
					g_LBTools_FBX.str_batchSourcePath = getSavePath caption:"Select a folder of animation maxfiles to export..." initialDir:maxFilePath
					if g_LBTools_FBX.str_batchSourcePath != undefined then
					(
						if chkbx_selectedMaxfiles.checked then --// Use a selection dialog to choose what animations to export
						(
							g_LBTools_FBX.f_selectMaxfilesUI() --// This function runs the f_batchExportAnimations function when the selection dialog export button is pressed
						)
						else --// Export all the animations in the chosen path
						(
							if chkbx_recurseSubfolders.checked then
							(
								g_LBTools_FBX.a_maxfilesToBatch = g_LBTools_FBX.f_getFilesRecursive g_LBTools_FBX.str_batchSourcePath "*.max"
							)
							else
							(
								g_LBTools_FBX.a_maxfilesToBatch = getFiles (g_LBTools_FBX.str_batchSourcePath + "\*.max")
							)
							g_LBTools_FBX.f_batchExportAnimations g_LBTools_FBX.a_maxfilesToBatch
						)
					)
					else
					(
						messageBox "Batch animation export process aborted." title:""
					)
				)
			)
		)
	) --// end rollout
	
	rollout about_Rollout "About"
	(
		label lbl_author "Author - Lorne Brooks"
		hyperLink myEmail "lbtools2k@gmail.com" color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
		label lbl_version "Version 2.4 - 24 Sept 2022"
	) --// end rollout
	
	fn f_openExporterUI =
	(
		try(destroyDialog g_LBTools_FBX.exportFBX_Rollout)catch()
		if g_LBTools_FBX.dPos == undefined do
		(
			g_LBTools_FBX.dPos = mouse.screenpos + [-75,-15]
		)
		createDialog g_LBTools_FBX.exportFBX_Rollout width:450 pos:g_LBTools_FBX.dPos
	)
	
	g_LBTools_FBX.f_openExporterUI()
) --// end function

) --// end struct

g_LBTools_FBX = s_LBTools_ExportFBX2Unreal()


/*********************************************************************************************************************************************************************/
--// MACRO CALLS FOR UI
/*********************************************************************************************************************************************************************/

macroscript exportFBX_Unreal
	category:"LB Tools"
	buttontext:"Export FBX - Unreal 2.4"
	icon:#("LBTools_FBX",1)
	tooltip:"Export FBX - Unreal 2.4"
(
	g_LBTools_FBX.f_initializeFBXexport()
	if keyboard.shiftPressed then --// Allow quick Static or Skeletal exporting based on the UI setting
	(
		if (doesFileExist g_LBTools_FBX.exporterINI) then
		(
			local str_shiftClick = getIniSetting g_LBTools_FBX.exporterINI "FBX Export" "ShiftClick"
			if str_shiftClick == "1" then --// STATIC mesh export
			(
				if maxFilePath != "" then
				(
					g_LBTools_FBX.f_initializeFBXexport()
					holdMaxFile()
					unhide objects doLayer:true
					unfreeze objects doLayer:true
					g_LBTools_FBX.f_exportFBX_Static()
					fetchMaxFile quiet:true
				)
				else
				(
					messagebox "Error: Scene not saved to a file. Open or save a file to export from." title:""
				)
			)
			else --// SKELETAL mesh export
			(
				if maxFilePath != "" then
				(
					g_LBTools_FBX.f_initializeFBXexport()
					holdMaxFile()
					unhide objects doLayer:true
					unfreeze objects doLayer:true
					g_LBTools_FBX.f_exportFBX_Skeletal()
					if g_LBTools_FBX.skinMeshSelectProcess == false do
					(
						fetchMaxFile quiet:true
					)
				)
				else
				(
					messagebox "Error: Scene not saved to a file. Open or save a file to export from." title:""
				)
			)
		)
		else
		(
			g_LBTools_FBX.f_exportFBX_UI()
		)
	)
	else --// Open the UI
	(
		g_LBTools_FBX.f_exportFBX_UI()
	)
)

/*********************************************************************************************************************************************************************/w